# Modifications Copyright(C)[2025] Advanced Micro Devices, Inc. All rights reserved.
# https://github.com/thunlp/TritonBench - Apache License 2.0
import triton
import triton.language as tl
import torch


@triton.jit
def _fwd_kernel_token_att2(
    Prob,
    V,
    Out,
    Req_to_tokens,
    B_req_idx,
    B_Start_Loc,
    B_Seqlen,
    B_Att_Start_Loc,
    B_Att_Seqlen,
    stride_req_to_tokens_b,
    stride_req_to_tokens_s,
    stride_ph,
    stride_pbs,
    stride_vbs,
    stride_vh,
    stride_vd,
    stride_obs,
    stride_oh,
    stride_od,
    kv_group_num,
    sliding_window,
    BLOCK_DMODEL: tl.constexpr,
    BLOCK_N: tl.constexpr,
):
    # Triton kernel for computing token attention
    cur_batch = tl.program_id(0)
    cur_head = tl.program_id(1)
    cur_kv_head = cur_head // kv_group_num

    offs_n = tl.arange(0, BLOCK_N)
    offs_d = tl.arange(0, BLOCK_DMODEL)
    cur_batch_seq_len = tl.load(B_Seqlen + cur_batch)
    cur_batch_start_index = tl.maximum(cur_batch_seq_len - sliding_window, 0)
    cur_batch_in_all_start_index = tl.load(B_Att_Start_Loc + cur_batch)
    cur_batch_req_idx = tl.load(B_req_idx + cur_batch)
    cur_att_seq_len = tl.load(B_Att_Seqlen + cur_batch)

    v_loc_off = (
        cur_batch_req_idx * stride_req_to_tokens_b + (cur_batch_start_index + offs_n) * stride_req_to_tokens_s
    )
    p_offs = cur_head * stride_ph + (cur_batch_in_all_start_index + offs_n) * stride_pbs
    v_offs = cur_kv_head * stride_vbs + offs_d[None, :] * stride_vd

    acc = tl.zeros([BLOCK_DMODEL], dtype=tl.float32)
    for start_n in range(0, cur_att_seq_len, BLOCK_N):
        start_n = tl.multiple_of(start_n, BLOCK_N)
        p_value = tl.load(Prob + p_offs + start_n, mask=(start_n + offs_n) < cur_att_seq_len, other=0.0)
        v_loc = tl.load(
            Req_to_tokens + v_loc_off + start_n * stride_req_to_tokens_s,
            mask=(start_n + offs_n + cur_batch_start_index) < cur_batch_seq_len,
            other=0.0,
        )
        v_value = tl.load(
            V + v_offs + v_loc[:, None] * stride_vh,
            mask=(start_n + offs_n[:, None] + cur_batch_start_index) < cur_batch_seq_len,
            other=0.0,
        )
        acc += tl.sum(p_value[:, None] * v_value, 0)

    acc = acc.to(Out.dtype.element_ty)
    off_o = cur_batch * stride_obs + cur_head * stride_oh + offs_d * stride_od
    out_ptrs = Out + off_o
    tl.store(out_ptrs, acc)
    return


@torch.no_grad()
def token_att_fwd2(
    prob, v, out, Req_to_tokens, B_req_idx, B_Start_Loc, B_Seqlen, B_Att_Start_Loc, B_Att_Seqlen, sliding_window
):
    # Launch the Triton kernel for token attention
    BLOCK = 128
    batch, head = B_req_idx.shape[0], prob.shape[0]
    grid = (batch, head)
    num_warps = 4
    dim = v.shape[-1]

    kv_group_num = prob.shape[0] // v.shape[1]

    _fwd_kernel_token_att2[grid](
        prob,
        v,
        out,
        Req_to_tokens,
        B_req_idx,
        B_Start_Loc,
        B_Seqlen,
        B_Att_Start_Loc,
        B_Att_Seqlen,
        Req_to_tokens.stride(0),
        Req_to_tokens.stride(1),
        prob.stride(0),
        prob.stride(1),
        v.stride(0),
        v.stride(1),
        v.stride(2),
        out.stride(0),
        out.stride(1),
        out.stride(2),
        kv_group_num=kv_group_num,
        sliding_window=sliding_window,
        BLOCK_DMODEL=dim,
        BLOCK_N=BLOCK,
        num_warps=num_warps,
        num_stages=1,
    )
    return




##################################################################################################################################################


# Define the test function
def test_token_att_fwd2():
    # Define the dimensions
    batch_size = 2
    num_heads = 4
    seq_len = 128
    d_model = 64
    sliding_window = 64

    # Create random tensors for inputs
    prob = torch.rand((num_heads, seq_len), dtype=torch.float32, device='cuda')
    v = torch.rand((num_heads, seq_len, d_model), dtype=torch.float32, device='cuda')
    Req_to_tokens = torch.randint(0, seq_len, (batch_size, seq_len), dtype=torch.int32, device='cuda')
    B_req_idx = torch.randint(0, batch_size, (batch_size,), dtype=torch.int32, device='cuda')
    B_Start_Loc = torch.zeros((batch_size,), dtype=torch.int32, device='cuda')
    B_Seqlen = torch.full((batch_size,), seq_len, dtype=torch.int32, device='cuda')
    B_Att_Start_Loc = torch.zeros((batch_size,), dtype=torch.int32, device='cuda')
    B_Att_Seqlen = torch.full((batch_size,), seq_len, dtype=torch.int32, device='cuda')

    results = {}

    # Test case 1
    out1 = torch.zeros((batch_size, num_heads, d_model), dtype=torch.float32, device='cuda')
    token_att_fwd2(
        prob, v, out1, Req_to_tokens, B_req_idx, B_Start_Loc, B_Seqlen, B_Att_Start_Loc, B_Att_Seqlen, sliding_window
    )
    results['test_case_1'] = out1.clone()

    # Test case 2 (different sliding_window size)
    sliding_window = 32
    out2 = torch.zeros((batch_size, num_heads, d_model), dtype=torch.float32, device='cuda')
    token_att_fwd2(
        prob, v, out2, Req_to_tokens, B_req_idx, B_Start_Loc, B_Seqlen, B_Att_Start_Loc, B_Att_Seqlen, sliding_window
    )
    results['test_case_2'] = out2.clone()

    # Test case 3 (different sequence length for Req_to_tokens)
    Req_to_tokens = torch.randint(0, seq_len, (batch_size, seq_len // 2), dtype=torch.int32, device='cuda')
    out3 = torch.zeros((batch_size, num_heads, d_model), dtype=torch.float32, device='cuda')
    token_att_fwd2(
        prob, v, out3, Req_to_tokens, B_req_idx, B_Start_Loc, B_Seqlen, B_Att_Start_Loc, B_Att_Seqlen, sliding_window
    )
    results['test_case_3'] = out3.clone()

    # Test case 4 (different batch size)
    batch_size = 4
    prob = torch.rand((num_heads, seq_len), dtype=torch.float32, device='cuda')
    v = torch.rand((num_heads, seq_len, d_model), dtype=torch.float32, device='cuda')
    Req_to_tokens = torch.randint(0, seq_len, (batch_size, seq_len), dtype=torch.int32, device='cuda')
    B_req_idx = torch.randint(0, batch_size, (batch_size,), dtype=torch.int32, device='cuda')
    B_Start_Loc = torch.zeros((batch_size,), dtype=torch.int32, device='cuda')
    B_Seqlen = torch.full((batch_size,), seq_len, dtype=torch.int32, device='cuda')
    B_Att_Start_Loc = torch.zeros((batch_size,), dtype=torch.int32, device='cuda')
    B_Att_Seqlen = torch.full((batch_size,), seq_len, dtype=torch.int32, device='cuda')

    out4 = torch.zeros((batch_size, num_heads, d_model), dtype=torch.float32, device='cuda')
    token_att_fwd2(
        prob, v, out4, Req_to_tokens, B_req_idx, B_Start_Loc, B_Seqlen, B_Att_Start_Loc, B_Att_Seqlen, sliding_window
    )
    results['test_case_4'] = out4.clone()

    return results


# Execute the test function
result_gold = test_token_att_fwd2()



# Modifications Copyright(C)[2025] Advanced Micro Devices, Inc. All rights reserved.
# https://github.com/thunlp/TritonBench - Apache License 2.0
import torch
import triton
import triton.language as tl


@triton.jit
def iv_dependent_matmul_kernel(a_ptr, b_ptr, c_ptr,  #
                                M, N, K,  #
                                stride_am, stride_ak,  #
                                stride_bk, stride_bn,  #
                                stride_cm, stride_cn,  #
                                BLOCK_SIZE_M: tl.constexpr, BLOCK_SIZE_N: tl.constexpr, BLOCK_SIZE_K: tl.constexpr,  #
                                type: tl.constexpr):
    pid = tl.program_id(axis=0)
    num_pid_n = tl.cdiv(N, BLOCK_SIZE_N)
    pid_m = pid // num_pid_n
    pid_n = pid % num_pid_n

    offs_am = (pid_m * BLOCK_SIZE_M + tl.arange(0, BLOCK_SIZE_M)) % M
    offs_bn = (pid_n * BLOCK_SIZE_N + tl.arange(0, BLOCK_SIZE_N)) % N
    offs_k = tl.arange(0, BLOCK_SIZE_K)
    a_ptr = a_ptr + (offs_am[:, None] * stride_am + offs_k[None, :] * stride_ak)
    b_ptr = b_ptr + (offs_k[:, None] * stride_bk + offs_bn[None, :] * stride_bn)
    a_ptrs = a_ptr
    b_ptrs = b_ptr
    if type == "post_load_two_iters":
        a_ptrs_next = a_ptr + BLOCK_SIZE_K * stride_ak
        b_ptrs_next = b_ptr + BLOCK_SIZE_K * stride_bk
    elif type == "post_load_three_iters":
        a_ptrs_next = a_ptr + BLOCK_SIZE_K * stride_ak
        b_ptrs_next = b_ptr + BLOCK_SIZE_K * stride_bk
        a_ptrs_next_next = a_ptr + 2 * BLOCK_SIZE_K * stride_ak
        b_ptrs_next_next = b_ptr + 2 * BLOCK_SIZE_K * stride_bk

    accumulator = tl.zeros((BLOCK_SIZE_M, BLOCK_SIZE_N), dtype=tl.float32)
    for k in range(0, tl.cdiv(K, BLOCK_SIZE_K)):
        if type == "pre_load":
            a_ptrs = a_ptr + k * BLOCK_SIZE_K * stride_ak
            b_ptrs = b_ptr + k * BLOCK_SIZE_K * stride_bk
        elif type == "post_pre_mixed":
            a_ptrs = a_ptr + k * BLOCK_SIZE_K * stride_ak
        a = tl.load(a_ptrs, mask=offs_k[None, :] < K - k * BLOCK_SIZE_K, other=0.0)
        b = tl.load(b_ptrs, mask=offs_k[:, None] < K - k * BLOCK_SIZE_K, other=0.0)
        accumulator += tl.dot(a, b)
        if type == "post_load":
            a_ptrs = a_ptr + (k + 1) * BLOCK_SIZE_K * stride_ak
            b_ptrs = b_ptr + (k + 1) * BLOCK_SIZE_K * stride_bk
        elif type == "post_pre_mixed":
            b_ptrs = b_ptr + (k + 1) * BLOCK_SIZE_K * stride_bk
        elif type == "post_load_two_iters":
            a_ptrs = a_ptrs_next
            b_ptrs = b_ptrs_next
            a_ptrs_next = a_ptr + (k + 2) * BLOCK_SIZE_K * stride_ak
            b_ptrs_next = b_ptr + (k + 2) * BLOCK_SIZE_K * stride_bk
        elif type == "post_load_three_iters":
            a_ptrs = a_ptrs_next
            b_ptrs = b_ptrs_next
            a_ptrs_next = a_ptrs_next_next
            b_ptrs_next = b_ptrs_next_next
            a_ptrs_next_next = a_ptr + (k + 3) * BLOCK_SIZE_K * stride_ak
            b_ptrs_next_next = b_ptr + (k + 3) * BLOCK_SIZE_K * stride_bk
    c = accumulator.to(tl.float16)

    offs_cm = pid_m * BLOCK_SIZE_M + tl.arange(0, BLOCK_SIZE_M)
    offs_cn = pid_n * BLOCK_SIZE_N + tl.arange(0, BLOCK_SIZE_N)
    c_ptrs = c_ptr + stride_cm * offs_cm[:, None] + stride_cn * offs_cn[None, :]
    c_mask = (offs_cm[:, None] < M) & (offs_cn[None, :] < N)
    tl.store(c_ptrs, c, mask=c_mask)


def iv_dependent_matmul_wrapper(
    M: int, 
    K: int, 
    N: int, 
    BLOCK_SIZE_M: int, 
    BLOCK_SIZE_N: int, 
    BLOCK_SIZE_K: int, 
    type: str = "pre_load",  # Kernel type for scheduling ("pre_load", "post_load", etc.)
    device: torch.device = "cuda"  # Device to run the test (defaults to "cuda")
):
    # Ensure the device is correct
    device = torch.device(device)

    # Generate random input matrices a and b on the specified device
    a = torch.rand((M, K), device=device)
    b = torch.rand((K, N), device=device)

    # Create an empty tensor to store the Triton result
    triton_output = torch.empty((M, N), device=device)

    # Define Triton grid configuration
    def grid(META):
        return (triton.cdiv(M, META['BLOCK_SIZE_M']) * triton.cdiv(N, META['BLOCK_SIZE_N']), )

    # Set the number of stages based on the kernel type
    num_stages = 4 if type == "post_load_three_iters" else 3

    # Run the Triton kernel
    iv_dependent_matmul_kernel[grid](
        a, b, triton_output, M, N, K,  #
        a.stride(0), a.stride(1), b.stride(0), b.stride(1),  #
        triton_output.stride(0), triton_output.stride(1),  #
        BLOCK_SIZE_M=BLOCK_SIZE_M, BLOCK_SIZE_N=BLOCK_SIZE_N, BLOCK_SIZE_K=BLOCK_SIZE_K, type=type,  #
        num_stages=num_stages
    )

    # Optionally print the result for inspection
    # print(triton_output)

    return triton_output




##################################################################################################################################################


import torch

# 封装 IV Dependent MatMul 测试的函数
def test_iv_dependent_matmul_kernel():
    # 定义矩阵维度和块大小
    M = 256
    K = 256
    N = 256
    BLOCK_SIZE_M = 32
    BLOCK_SIZE_N = 32
    BLOCK_SIZE_K = 32

    # 创建 CUDA 设备
    device = torch.device('cuda')

    # 定义所有类型的内核配置
    types = [
        "pre_load",
        "post_load",
        "post_pre_mixed",
        "post_load_two_iters",
        "post_load_three_iters"
    ]

    # 字典用于存储每个测试用例的结果
    results = {}

    # 遍历每种内核类型，进行测试
    for i, type in enumerate(types):
        # 调用封装函数运行 Triton 核心
        triton_output = iv_dependent_matmul_wrapper(
            M=M,
            K=K,
            N=N,
            BLOCK_SIZE_M=BLOCK_SIZE_M,
            BLOCK_SIZE_N=BLOCK_SIZE_N,
            BLOCK_SIZE_K=BLOCK_SIZE_K,
            type=type,
            device=device
        )

        # 确保输出的大小正确
        assert triton_output.shape == (M, N), f"Expected output shape {(M, N)} but got {triton_output.shape} for type {type}"

        # 保存结果到字典
        results[f"test_case_{i+1}"] = triton_output

    return results

# 执行测试函数进行所有类型的验证
result_gold = test_iv_dependent_matmul_kernel()



import sys
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from TritonBench_v1.swiglu_backward import _swiglu_bwd
from performance_utils import Performance_Metrics, do_bench_config

import torch
import triton
import triton.language as tl

class performance_metrics(Performance_Metrics):
    def __init__(self, dtype=None, is_backward=False, **kwargs):
        super().__init__('swiglu_backward', dtype=dtype, is_backward=is_backward, **kwargs)
        
    def get_input_tensors(self):
        self.input_tensors = []
        for i in range(2, 14):
            size = 2 ** i
            xy = torch.rand((128, size * 2), dtype=torch.float32).reshape(1, -1)
            dout = torch.rand((128, size), dtype=torch.float32).reshape(1, -1)
            self.input_tensors.append((xy, dout))

    def to_cuda(self, input_tensor):
        xy, dout = input_tensor
        return (xy.cuda(), dout.cuda())

    def call_op(self, input_tensor):
        xy, dout = input_tensor
        return _swiglu_bwd(xy, dout)

    def get_gbps(self, input_tensor, runtime):
        xy, dout = input_tensor
        total_bytes = (xy.numel() + dout.numel()) * xy.element_size() * 2  # Read and write
        GBPS = total_bytes / (runtime / 1000) / 1e9
        return GBPS
    
    def get_tflops(self, input_tensor, runtime):
        xy, dout = input_tensor
        N = dout.numel()
        FLOPS = 5 * N  # Based on the operations in the kernel
        TFLOPS = FLOPS / (runtime / 1000) / 1e12
        return TFLOPS

if __name__ == '__main__':
    op_perf = performance_metrics()
    op_perf.get_input_tensors()
    op_perf.get_do_bench_config(warmup=100, rep=1000)
    op_perf.run_benchmark()



# Modifications Copyright(C)[2025] Advanced Micro Devices, Inc. All rights reserved.
# https://github.com/thunlp/TritonBench - Apache License 2.0
import math
import torch
import triton
import triton.language as tl

@triton.autotune(
    configs=[
        triton.Config({'BLOCK_SIZE_M': 128, 'BLOCK_SIZE_N': 256, 'BLOCK_SIZE_K': 64}, num_stages=3, num_warps=8),
        triton.Config({'BLOCK_SIZE_M': 64, 'BLOCK_SIZE_N': 256, 'BLOCK_SIZE_K': 32}, num_stages=4, num_warps=4),
        triton.Config({'BLOCK_SIZE_M': 128, 'BLOCK_SIZE_N': 128, 'BLOCK_SIZE_K': 32}, num_stages=4, num_warps=4),
        triton.Config({'BLOCK_SIZE_M': 128, 'BLOCK_SIZE_N': 64, 'BLOCK_SIZE_K': 32}, num_stages=4, num_warps=4),
        triton.Config({'BLOCK_SIZE_M': 64, 'BLOCK_SIZE_N': 128, 'BLOCK_SIZE_K': 32}, num_stages=4, num_warps=4),
        triton.Config({'BLOCK_SIZE_M': 128, 'BLOCK_SIZE_N': 32, 'BLOCK_SIZE_K': 32}, num_stages=4, num_warps=4),
        triton.Config({'BLOCK_SIZE_M': 64, 'BLOCK_SIZE_N': 32, 'BLOCK_SIZE_K': 32}, num_stages=5, num_warps=2),
        triton.Config({'BLOCK_SIZE_M': 32, 'BLOCK_SIZE_N': 64, 'BLOCK_SIZE_K': 32}, num_stages=5, num_warps=2),
        triton.Config({'BLOCK_SIZE_M': 64, 'BLOCK_SIZE_N': 64, 'BLOCK_SIZE_K': 32}, num_stages=4, num_warps=2),
    ],
    key=['chunk_size', 'K', 'IS_CAUSAL'],
)
@triton.jit
def _bmm_chunk_fwd_kernel(
    a_ptr, b_ptr, out_ptr, seq_idx_ptr,
    seqlen, chunk_size, K, ngroups,
    stride_a_batch, stride_a_seqlen, stride_a_head, stride_ak,
    stride_b_batch, stride_b_seqlen, stride_b_head, stride_bk,
    stride_out_batch, stride_out_chunk, stride_out_head, stride_outm, stride_outn,
    stride_seq_idx_batch, stride_seq_idx_seqlen,
    IS_CAUSAL: tl.constexpr,
    dot_dtype: tl.constexpr,
    HAS_SEQ_IDX: tl.constexpr,
    BLOCK_SIZE_M: tl.constexpr, BLOCK_SIZE_N: tl.constexpr, BLOCK_SIZE_K: tl.constexpr,
):
    pid_b = tl.program_id(axis=1)
    pid_ch = tl.program_id(axis=2)
    pid_c = pid_ch // ngroups
    pid_h = pid_ch - pid_c * ngroups
    num_pid_n = tl.cdiv(chunk_size, BLOCK_SIZE_N)
    pid_m = tl.program_id(axis=0) // num_pid_n
    pid_n = tl.program_id(axis=0) % num_pid_n
    if IS_CAUSAL:
        if pid_n * BLOCK_SIZE_N >= (pid_m + 1) * BLOCK_SIZE_M:
            return
    a_ptr += pid_b * stride_a_batch + pid_c * chunk_size * stride_a_seqlen + pid_h * stride_a_head
    b_ptr += pid_b * stride_b_batch + pid_c * chunk_size * stride_b_seqlen + pid_h * stride_b_head
    if HAS_SEQ_IDX:
        seq_idx_ptr += pid_b * stride_seq_idx_batch + pid_c * chunk_size * stride_seq_idx_seqlen

    offs_m = pid_m * BLOCK_SIZE_M + tl.arange(0, BLOCK_SIZE_M)
    offs_n = pid_n * BLOCK_SIZE_N + tl.arange(0, BLOCK_SIZE_N)
    offs_k = tl.arange(0, BLOCK_SIZE_K)
    a_ptrs = a_ptr + (offs_m[:, None] * stride_a_seqlen + offs_k[None, :] * stride_ak)
    b_ptrs = b_ptr + (offs_k[:, None] * stride_bk + offs_n[None, :] * stride_b_seqlen)
    chunk_size_limit = min(chunk_size, seqlen - pid_c * chunk_size)

    acc = tl.zeros((BLOCK_SIZE_M, BLOCK_SIZE_N), dtype=tl.float32)
    for k in range(0, tl.cdiv(K, BLOCK_SIZE_K)):
        a = tl.load(a_ptrs, mask=(offs_m[:, None] < chunk_size_limit) & (offs_k[None, :] < K - k * BLOCK_SIZE_K), other=0.0).to(dot_dtype)
        b = tl.load(b_ptrs, mask=(offs_k[:, None] < K - k * BLOCK_SIZE_K) & (offs_n[None, :] < chunk_size_limit), other=0.0).to(dot_dtype)
        acc += tl.dot(a, b)
        a_ptrs += BLOCK_SIZE_K * stride_ak
        b_ptrs += BLOCK_SIZE_K * stride_bk

    offs_m = pid_m * BLOCK_SIZE_M + tl.arange(0, BLOCK_SIZE_M)
    offs_n = pid_n * BLOCK_SIZE_N + tl.arange(0, BLOCK_SIZE_N)
    if HAS_SEQ_IDX:
        chunk_size_limit = min(chunk_size, seqlen - pid_c * chunk_size)
        seq_idx_m = tl.load(seq_idx_ptr + offs_m * stride_seq_idx_seqlen, mask=offs_m < chunk_size_limit, other=-1)
        seq_idx_n = tl.load(seq_idx_ptr + offs_n * stride_seq_idx_seqlen, mask=offs_n < chunk_size_limit, other=-2)
        acc = tl.where(seq_idx_m[:, None] == seq_idx_n[None, :], acc, 0.0)
    out = acc.to(out_ptr.dtype.element_ty)

    out_ptr += pid_b * stride_out_batch + pid_c * stride_out_chunk + pid_h * stride_out_head
    out_ptrs = out_ptr + (stride_outm * offs_m[:, None] + offs_n[None, :] * stride_outn)
    tl.store(out_ptrs, out, mask=(offs_m[:, None] < chunk_size) & (offs_n[None, :] < chunk_size))

def _bmm_chunk_fwd(a, b, chunk_size, seq_idx=None, causal=False, output_dtype=None):
    has_groups = a.dim() == 4
    if not has_groups:
        batch, seqlen, k = a.shape
    else:
        batch, seqlen, ngroups, k = a.shape
    assert b.shape == a.shape
    if seq_idx is not None:
        assert seq_idx.shape == (batch, seqlen)
    if a.stride(-1) != 1 and a.stride(1) != 1:
        a = a.contiguous()
    if b.stride(-1) != 1 and b.stride(1) != 1:
        b = b.contiguous()
    nchunks = math.ceil(seqlen / chunk_size)
    out_dtype = a.dtype if output_dtype is None else output_dtype
    out = torch.empty((batch, nchunks, chunk_size, chunk_size) if not has_groups else (batch, nchunks, ngroups, chunk_size, chunk_size),
                      device=a.device, dtype=out_dtype)
    dot_dtype = (tl.bfloat16 if a.dtype == torch.bfloat16 or b.dtype == torch.bfloat16 else
                 (tl.float16 if a.dtype == torch.float16 or b.dtype == torch.float16 else tl.float32))
    grid = lambda META: (triton.cdiv(chunk_size, META['BLOCK_SIZE_M']) * triton.cdiv(chunk_size, META['BLOCK_SIZE_N']),
                    batch, nchunks if not has_groups else nchunks * ngroups)
    with torch.cuda.device(a.device.index):
        _bmm_chunk_fwd_kernel[grid](
            a, b, out, seq_idx,
            int(seqlen), int(chunk_size), int(k), int(ngroups if has_groups else 1),
            a.stride(0), a.stride(1), 0 if not has_groups else a.stride(2), a.stride(-1),
            b.stride(0), b.stride(1), 0 if not has_groups else b.stride(2), b.stride(-1),
            out.stride(0), out.stride(1), 0 if not has_groups else out.stride(2), out.stride(-2), out.stride(-1),
            *((seq_idx.stride(0), seq_idx.stride(1)) if seq_idx is not None else (0, 0)),
            causal,
            dot_dtype,
            HAS_SEQ_IDX=seq_idx is not None,
        )
    return out



##################################################################################################################################################


import torch

# Test for _bmm_chunk_fwd
def test_bmm_chunk_fwd():
    results = {}
    
    # Test case 1: Without groups, no seq_idx, not causal
    a = torch.randn(2, 128, 64, device='cuda', dtype=torch.float16)
    b = torch.randn(2, 128, 64, device='cuda', dtype=torch.float16)
    chunk_size = 32
    out = _bmm_chunk_fwd(a, b, chunk_size)
    results['test_case_1'] = out.shape

    # Test case 2: With groups, with seq_idx, causal
    a = torch.randn(2, 128, 4, 64, device='cuda', dtype=torch.float16)
    b = torch.randn(2, 128, 4, 64, device='cuda', dtype=torch.float16)
    seq_idx = torch.arange(128, device='cuda').repeat(2, 1)
    out = _bmm_chunk_fwd(a, b, chunk_size, seq_idx=seq_idx, causal=True)
    results['test_case_2'] = out.shape

    # Test case 3: Without groups, with seq_idx, not causal
    a = torch.randn(2, 128, 64, device='cuda', dtype=torch.float16)
    b = torch.randn(2, 128, 64, device='cuda', dtype=torch.float16)
    seq_idx = torch.arange(128, device='cuda').repeat(2, 1)
    out = _bmm_chunk_fwd(a, b, chunk_size, seq_idx=seq_idx, causal=False)
    results['test_case_3'] = out.shape

    # Test case 4: With groups, no seq_idx, not causal
    a = torch.randn(2, 128, 4, 64, device='cuda', dtype=torch.float16)
    b = torch.randn(2, 128, 4, 64, device='cuda', dtype=torch.float16)
    out = _bmm_chunk_fwd(a, b, chunk_size, causal=False)
    results['test_case_4'] = out.shape

    return results

# Run tests
result_gold = test_bmm_chunk_fwd()



# Modifications Copyright(C)[2025] Advanced Micro Devices, Inc. All rights reserved.
# https://github.com/thunlp/TritonBench - Apache License 2.0
import warnings
from typing import Optional

import torch
import triton
import triton.language as tl


@triton.jit
def decoding_fused_rotary_embedding_kernel(
    q,
    k,
    v,
    cos,
    sin,
    k_cache,
    v_cache,
    BLOCK_TABLES,
    context_lengths,
    x,
    q_token_stride,
    q_head_stride,
    k_token_stride,
    k_head_stride,
    head_dim_stride,
    cos_token_stride,
    cos_stride,
    kcb_stride,
    kch_stride,
    kcsplit_x_stride,
    kcs_stride,
    kcd_stride,
    vcb_stride,
    vch_stride,
    vcs_stride,
    vcd_stride,
    bts_stride,
    btb_stride,
    block_size,
    KV_GROUP_NUM: tl.constexpr,
    HEAD_DIM: tl.constexpr,
):
    cur_head_idx = tl.program_id(0)
    cur_token_idx = tl.program_id(1)

    dim_range = tl.arange(0, HEAD_DIM)
    dim_range0 = tl.arange(0, HEAD_DIM // 2)
    dim_range1 = tl.arange(HEAD_DIM // 2, HEAD_DIM)

    off_q = cur_token_idx * q_token_stride + cur_head_idx * q_head_stride
    off_q0 = off_q + dim_range0 * head_dim_stride
    off_q1 = off_q + dim_range1 * head_dim_stride

    loaded_q0 = tl.load(q + off_q0)
    loaded_q1 = tl.load(q + off_q1)
    off_cos_sin = cur_token_idx * cos_token_stride + dim_range0 * cos_stride
    loaded_cos = tl.load(cos + off_cos_sin)
    loaded_sin = tl.load(sin + off_cos_sin)

    out_q0 = loaded_q0 * loaded_cos - loaded_q1 * loaded_sin
    out_q1 = loaded_q0 * loaded_sin + loaded_q1 * loaded_cos
    tl.store(q + off_q0, out_q0)
    tl.store(q + off_q1, out_q1)

    handle_kv = cur_head_idx % KV_GROUP_NUM == 0
    if handle_kv:
        cur_k_head_idx = cur_head_idx // KV_GROUP_NUM
        off_kv = cur_token_idx * k_token_stride + cur_k_head_idx * k_head_stride
        off_k0 = off_kv + dim_range0 * head_dim_stride
        off_k1 = off_kv + dim_range1 * head_dim_stride
        loaded_k0 = tl.load(k + off_k0)
        loaded_k1 = tl.load(k + off_k1)

        out_k0 = loaded_k0 * loaded_cos - loaded_k1 * loaded_sin
        out_k1 = loaded_k0 * loaded_sin + loaded_k1 * loaded_cos

        # NOTE The precondition here is that it's only for unpadded inputs during decoding stage,
        # and so that we could directly use the token index as the sequence index
        past_kv_seq_len = tl.load(context_lengths + cur_token_idx) - 1

        last_block_idx = past_kv_seq_len // block_size
        block_ids = tl.load(BLOCK_TABLES + cur_token_idx * bts_stride + last_block_idx * btb_stride)
        offsets_in_last_block = past_kv_seq_len % block_size
        offsets_cache_base = block_ids * kcb_stride + cur_k_head_idx * kch_stride
        k_range0 = (
            offsets_cache_base
            + offsets_in_last_block * kcs_stride
            + (dim_range0 // x) * kcsplit_x_stride
            + (dim_range0 % x) * kcd_stride
        )
        k_range1 = (
            offsets_cache_base
            + offsets_in_last_block * kcs_stride
            + (dim_range1 // x) * kcsplit_x_stride
            + (dim_range1 % x) * kcd_stride
        )
        tl.store(k_cache + k_range0, out_k0)
        tl.store(k_cache + k_range1, out_k1)

        off_v = off_kv + dim_range * head_dim_stride
        loaded_v = tl.load(v + off_v)
        v_range = (
            block_ids * vcb_stride
            + cur_k_head_idx * vch_stride
            + offsets_in_last_block * vcs_stride
            + dim_range * vcd_stride
        )
        tl.store(v_cache + v_range, loaded_v)


def decoding_fused_rotary_embedding(
    q: torch.Tensor,
    k: torch.Tensor,
    v: torch.Tensor,
    cos: torch.Tensor,
    sin: torch.Tensor,
    k_cache: Optional[torch.Tensor] = None,
    v_cache: Optional[torch.Tensor] = None,
    block_tables: Optional[torch.Tensor] = None,
    kv_lengths: Optional[torch.Tensor] = None,
    use_new_kcache_layout: bool = False,
):
    """
    Args:
        q: query tensor, [total_tokens, head_num, head_dim]
        k: key tensor, [total_tokens, kv_head_num, head_dim]
        v: value tensor, [total tokens, kv_head_num, head_dim]
        cos: cosine for rotary embedding, [max_position_len, head_dim]
        sin: sine for rotary embedding, [max_position_len, head_dim]
        k_cache (torch.Tensor):  Blocked key cache. [num_blocks, kv_head_num, block_size, head_dim]
        v_cache (torch.Tensor):  Blocked value cache. [num_blocks, kv_head_num, block_size, head_dim]
        kv_lengths, Past key/value sequence lengths plus current sequence length for each sequence. [bsz]
        block_tables: Block tables for each sequence. [bsz, max_blocks_per_sequence]
    """
    q_total_tokens, q_head_num, head_dim = q.shape
    assert q.size(0) == k.size(0) == v.size(0)

    if head_dim >= 512:
        num_warps = 16
    elif head_dim >= 256:
        num_warps = 8
    else:
        num_warps = 4
    k_head_num = k.size(1)
    kv_group_num = q_head_num // k_head_num

    # For KCache and VCache with the same layout
    x = head_dim
    kcsplit_x_stride, kcs_stride, kcd_stride = 0, k_cache.stride(2), k_cache.stride(3)
    # For KCache layout [num_blocks, num_kv_heads, head_dim//x, block_size, x]
    if use_new_kcache_layout:
        assert (
            k_cache.dim() == 5
            and k_cache.shape[1] == v_cache.shape[1]
            and k_cache.shape[2] * k_cache.shape[4] == v_cache.shape[3]
        ), f"Invalid KCache shape {k_cache.shape} and VCache shape {v_cache.shape}"
        x = k_cache.size(-1)
        kcsplit_x_stride, kcs_stride, kcd_stride = k_cache.stride()[-3:]

    grid = (q_head_num, q_total_tokens)
    decoding_fused_rotary_embedding_kernel[grid](
        q,
        k,
        v,
        cos,
        sin,
        k_cache,
        v_cache,
        block_tables,
        kv_lengths,
        x,
        q.stride(0),
        q.stride(1),
        k.stride(0),
        k.stride(1),
        q.stride(2),
        cos.stride(0),
        cos.stride(1),
        k_cache.stride(0),
        k_cache.stride(1),
        kcsplit_x_stride,
        kcs_stride,
        kcd_stride,
        v_cache.stride(0),
        v_cache.stride(1),
        v_cache.stride(2),
        v_cache.stride(3),
        block_tables.stride(0),
        block_tables.stride(1),
        k_cache.size(-2),
        KV_GROUP_NUM=kv_group_num,
        HEAD_DIM=head_dim,
        num_warps=num_warps,
    )
    return




##################################################################################################################################################


def test_decoding_fused_rotary_embedding():
    # 定义测试参数
    total_tokens = 16       # 总 token 数
    q_head_num = 8          # Query 的头数量
    kv_head_num = 4         # Key/Value 的头数量
    head_dim = 64           # 每个头的维度
    max_position_len = 128  # 最大位置长度
    block_size = 4          # 块大小
    num_blocks = 4          # Key/Value cache 块数量
    batch_size = 2          # 批大小

    # 初始化输入张量
    q = torch.randn((total_tokens, q_head_num, head_dim), dtype=torch.float32, device='cuda')  # Query
    k = torch.randn((total_tokens, kv_head_num, head_dim), dtype=torch.float32, device='cuda')  # Key
    v = torch.randn((total_tokens, kv_head_num, head_dim), dtype=torch.float32, device='cuda')  # Value
    cos = torch.randn((max_position_len, head_dim), dtype=torch.float32, device='cuda')  # Cosine
    sin = torch.randn((max_position_len, head_dim), dtype=torch.float32, device='cuda')  # Sine

    # 初始化 Key/Value 缓存和辅助张量
    k_cache = torch.zeros((num_blocks, kv_head_num, block_size, head_dim), dtype=torch.float32, device='cuda')
    v_cache = torch.zeros((num_blocks, kv_head_num, block_size, head_dim), dtype=torch.float32, device='cuda')
    block_tables = torch.randint(0, num_blocks, (batch_size, num_blocks), dtype=torch.int32, device='cuda')
    kv_lengths = torch.randint(1, total_tokens, (batch_size,), dtype=torch.int32, device='cuda')

    results = {}

    # 测试默认 k_cache 布局
    decoding_fused_rotary_embedding(
        q=q,
        k=k,
        v=v,
        cos=cos,
        sin=sin,
        k_cache=k_cache,
        v_cache=v_cache,
        block_tables=block_tables,
        kv_lengths=kv_lengths,
        use_new_kcache_layout=False,
    )
    results['test_case_1'] = {
        'q_shape': q.shape,
        'k_cache_shape': k_cache.shape,
        'v_cache_shape': v_cache.shape
    }

    # 测试新的 k_cache 布局
    x = 16  # 分割因子
    k_cache = torch.zeros((num_blocks, kv_head_num, head_dim // x, block_size, x), dtype=torch.float32, device='cuda')
    v_cache = torch.zeros((num_blocks, kv_head_num, block_size, head_dim), dtype=torch.float32, device='cuda')

    # 测试新的 k_cache 布局
    decoding_fused_rotary_embedding(
        q=q,
        k=k,
        v=v,
        cos=cos,
        sin=sin,
        k_cache=k_cache,
        v_cache=v_cache,
        block_tables=block_tables,
        kv_lengths=kv_lengths,
        use_new_kcache_layout=True,
    )
    results['test_case_2'] = {
        'q_shape': q.shape,
        'k_cache_shape': k_cache.shape,
        'v_cache_shape': v_cache.shape
    }

    return results

result_gold = test_decoding_fused_rotary_embedding()




# Modifications Copyright(C)[2025] Advanced Micro Devices, Inc. All rights reserved.
# https://github.com/thunlp/TritonBench - Apache License 2.0
import torch
import triton
import triton.language as tl

@triton.jit
def softmax_kernel(
    output_ptr, input_ptr, input_row_stride, output_row_stride, n_cols,
    BLOCK_SIZE: tl.constexpr,
):
    # Get the program ID
    row_idx = tl.program_id(axis=0)

    # Compute the memory offsets for this row
    row_start_ptr = input_ptr + row_idx * input_row_stride
    out_row_start_ptr = output_ptr + row_idx * output_row_stride

    # Load the row into SRAM
    row = tl.load(row_start_ptr + tl.arange(0, BLOCK_SIZE), mask=tl.arange(0, BLOCK_SIZE) < n_cols, other=-float('inf'))

    # Compute max for numerical stability
    row_max = tl.max(row, axis=0)
    
    # Subtract max from row and exponentiate
    numerator = tl.exp(row - row_max)
    
    # Compute sum for normalization
    denominator = tl.sum(numerator, axis=0)
    
    # Normalize
    softmax_output = numerator / denominator
    
    # Store the output
    tl.store(out_row_start_ptr + tl.arange(0, BLOCK_SIZE), softmax_output, mask=tl.arange(0, BLOCK_SIZE) < n_cols)

def triton_softmax(x):
    n_rows, n_cols = x.shape
    output = torch.empty_like(x)
    
    # Determine the block size
    BLOCK_SIZE = triton.next_power_of_2(n_cols)
    BLOCK_SIZE = min(BLOCK_SIZE, 1024)  
    
    # Launch the Triton kernel
    grid = (n_rows,)
    softmax_kernel[grid](
        output, x,
        x.stride(0), output.stride(0),
        n_cols, BLOCK_SIZE=BLOCK_SIZE
    )
    return output




##################################################################################################################################################


# Test cases for the triton_softmax function
def test_triton_softmax():
    results = {}
    
    # Test case 1: Simple 2x2 matrix
    x1 = torch.tensor([[1.0, 2.0], [3.0, 4.0]], dtype=torch.float32, device="cuda")
    output1 = triton_softmax(x1)
    results['test_case_1'] = output1

    # Test case 2: 3x3 matrix with negative values
    x2 = torch.tensor([[-1.0, -2.0, -3.0], [0.0, 0.0, 0.0], [1.0, 2.0, 3.0]], dtype=torch.float32, device="cuda")
    output2 = triton_softmax(x2)
    results['test_case_2'] = output2

    # Test case 3: 4x4 matrix with larger values
    x3 = torch.tensor([[10.0, 20.0, 30.0, 40.0], [5.0, 15.0, 25.0, 35.0], [0.0, 0.0, 0.0, 0.0], [-10.0, -20.0, -30.0, -40.0]], dtype=torch.float32, device="cuda")
    output3 = triton_softmax(x3)
    results['test_case_3'] = output3

    # Test case 4: 1x5 matrix (single row)
    x4 = torch.tensor([[1.0, 2.0, 3.0, 4.0, 5.0]], dtype=torch.float32, device="cuda")
    output4 = triton_softmax(x4)
    results['test_case_4'] = output4

    # Test case 5: 5x1 matrix (single column)
    x5 = torch.tensor([[1.0], [2.0], [3.0], [4.0], [5.0]], dtype=torch.float32, device="cuda")
    output5 = triton_softmax(x5)
    results['test_case_5'] = output5

    return results

result_gold = test_triton_softmax()



# Modifications Copyright(C)[2025] Advanced Micro Devices, Inc. All rights reserved.
# https://github.com/thunlp/TritonBench - Apache License 2.0
import torch
import triton
import triton.language as tl

@triton.jit
def _fwd_kernel_destindex_copy_quantize_kv(
    K, Dest_loc, Out, Out_scale,
    stride_k_bs, stride_k_h, stride_k_d,
    stride_o_bs, stride_o_h, stride_o_d,
    stride_os_bs, stride_os_h, stride_os_d,
    head_num,
    BLOCK_DMODEL: tl.constexpr,
    BLOCK_HEAD: tl.constexpr
):
    cur_index = tl.program_id(0)
    offs_h = tl.arange(0, BLOCK_HEAD)
    offs_d = tl.arange(0, BLOCK_DMODEL)

    dest_index = tl.load(Dest_loc + cur_index)
    src_data = tl.load(K + cur_index * stride_k_bs + offs_h[:, None] * stride_k_h + stride_k_d * offs_d[None, :], 
                       mask=offs_h[:, None] < head_num, other=0.0)
    abs_data = tl.abs(src_data)
    data_scale = (tl.max(abs_data, axis=1) / 127.).to(tl.float16)[:, None]
    q_src_data = (src_data / data_scale).to(tl.int8)
    o_ptrs = Out + dest_index * stride_o_bs + stride_o_h * offs_h[:, None] + stride_o_d * offs_d[None, :]
    os_ptrs = Out_scale + dest_index * stride_os_bs + stride_os_h * offs_h[:, None]
    tl.store(o_ptrs, q_src_data, mask=offs_h[:, None] < head_num)
    tl.store(os_ptrs, data_scale, mask=offs_h[:, None] < head_num)


@torch.no_grad()
def destindex_copy_quantize_kv(K, DestLoc, Out, Out_scale):
    seq_len = DestLoc.shape[0]
    head_num = K.shape[1]
    head_dim = K.shape[2]
    assert K.shape[1] == Out.shape[1] and K.shape[2] == Out.shape[2]
    BLOCK_HEAD = triton.next_power_of_2(head_num)
    grid = (seq_len,)
    num_warps = 1

    _fwd_kernel_destindex_copy_quantize_kv[grid](
        K, DestLoc, Out, Out_scale,
        K.stride(0), K.stride(1), K.stride(2),
        Out.stride(0), Out.stride(1), Out.stride(2),
        Out_scale.stride(0), Out_scale.stride(1), Out_scale.stride(2),
        head_num,
        BLOCK_DMODEL=head_dim,
        BLOCK_HEAD=BLOCK_HEAD,
        num_warps=num_warps,
        num_stages=1,
    )
    return



##################################################################################################################################################


def test_destindex_copy_quantize_kv():
    B, N_CTX, H, D = 16, 512, 8, 64
    
    # Test case 1
    src1 = torch.randn((B * N_CTX, H, D), dtype=torch.float16).cuda()
    dest_loc1 = torch.arange(0, B * N_CTX, dtype=torch.int32).cuda()
    value_dest1 = torch.randn((B * N_CTX, H, D), dtype=torch.float16).cuda().to(torch.int8)
    scale_dest1 = torch.randn((B * N_CTX, H, 1), dtype=torch.float16).cuda()
    destindex_copy_quantize_kv(src1, dest_loc1, value_dest1, scale_dest1)

    # Test case 2 - Randomized location indices
    dest_loc2 = torch.randint(0, B * N_CTX, (B * N_CTX,), dtype=torch.int32).cuda()
    src2 = torch.randn((B * N_CTX, H, D), dtype=torch.float16).cuda()
    value_dest2 = torch.randn((B * N_CTX, H, D), dtype=torch.float16).cuda().to(torch.int8)
    scale_dest2 = torch.randn((B * N_CTX, H, 1), dtype=torch.float16).cuda()
    destindex_copy_quantize_kv(src2, dest_loc2, value_dest2, scale_dest2)

    # Test case 4 - Larger head dimension
    D4 = 256
    src3 = torch.randn((B * N_CTX, H, D4), dtype=torch.float16).cuda()
    dest_loc3 = torch.arange(0, B * N_CTX, dtype=torch.int32).cuda()
    value_dest3 = torch.randn((B * N_CTX, H, D4), dtype=torch.float16).cuda().to(torch.int8)
    scale_dest3 = torch.randn((B * N_CTX, H, 1), dtype=torch.float16).cuda()
    destindex_copy_quantize_kv(src3, dest_loc3, value_dest3, scale_dest3)

    results = {
        "test_case_1": value_dest1,
        "test_case_2": value_dest2,
        "test_case_3": value_dest3,
    }

    return results

result_gold = test_destindex_copy_quantize_kv()



# Modifications Copyright(C)[2025] Advanced Micro Devices, Inc. All rights reserved.

# Copyright (c) 2023 NVIDIA Corporation & Affiliates. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

######################################## Imports ######################################## 

import pytest
import torch
from torch.testing import assert_close

import triton
import triton.language as tl
######################################## Imports ######################################## 


@triton.jit
def matmul_tma_load_store(  #
        a_ptr, b_ptr, c_ptr,  #
        M, N, K,  #
        stride_am, stride_ak,  #
        stride_bk, stride_bn,  #
        stride_cm, stride_cn,  #
        BLOCK_M: tl.constexpr, BLOCK_N: tl.constexpr, BLOCK_K: tl.constexpr,  #
        OUTPUT_F16: tl.constexpr  #
):
    a_block_ptr = tl.make_block_ptr(base=a_ptr, shape=(M, K), strides=(stride_am, stride_ak), offsets=(0, 0),
                                    block_shape=(BLOCK_M, BLOCK_K), order=(1, 0))
    b_block_ptr = tl.make_block_ptr(base=b_ptr, shape=(K, N), strides=(stride_bk, stride_bn), offsets=(0, 0),
                                    block_shape=(BLOCK_K, BLOCK_N), order=(0, 1))
    c_block_ptr = tl.make_block_ptr(base=c_ptr, shape=(M, N), strides=(stride_cm, stride_cn), offsets=(0, 0),
                                    block_shape=(BLOCK_M, BLOCK_N), order=(1, 0))
    a = tl.load(a_block_ptr)
    b = tl.load(b_block_ptr)

    c = tl.dot(a, b)
    if OUTPUT_F16:
        c = c.to(tl.float16)

    tl.store(c_block_ptr, c)

##################################################################################################################################################  


import pytest
import numpy as np
import random
import torch 
import os
import pytest
from torch.testing import assert_close
from geak_eval.perf.ROCm.performance_utils_pytest import PytestBenchmarker, do_bench_config, save_all_benchmark_results
from typing import Dict


result_gold = {}

######################################## HELPERS for Eval ######################################## 
def set_seed(seed: int = 42) -> None:
    """
    Set the random seed for reproducibility across multiple libraries and configure PyTorch for deterministic behavior.

    Args:
        seed (int): The seed value to set. Default is 42.
    """
    # Set seed for Python's built-in random module
    random.seed(seed)
    # Set seed for NumPy
    np.random.seed(seed)
    # Set seed for PyTorch on CPU
    torch.manual_seed(seed)
    # Set seed for PyTorch on all GPUs (if available)
    if torch.cuda.is_available():
        torch.cuda.manual_seed(seed)
        torch.cuda.manual_seed_all(seed)
    # Ensure deterministic behavior in PyTorch
    torch.backends.cudnn.deterministic = True
    torch.backends.cudnn.benchmark = False
    # Set environment variable for hash-based operations
    os.environ['PYTHONHASHSEED'] = str(seed)

######################################## HELPERS for Eval ######################################## 



@pytest.mark.parametrize('M,N,K,NUM_CTAS,NUM_WARPS,TRANS_A,TRANS_B,OUTPUT_F16', [
    [64, 64, 16, 1, 4, False, True, False],
    [64, 64, 16, 1, 4, False, True, True],
    [128, 64, 32, 1, 4, False, True, False],
    [128, 64, 32, 1, 4, False, True, True],
    [64, 128, 32, 1, 4, False, True, False],
    [64, 128, 32, 1, 4, False, True, True],
    [128, 128, 64, 1, 4, False, True, False],
    [128, 128, 64, 1, 4, False, True, True],
])
def test_tma_load_store(M, N, K, NUM_CTAS, NUM_WARPS, TRANS_A, TRANS_B, OUTPUT_F16, request):
    set_seed()

    if (TRANS_A):
        a = torch.randn((K, M), device='cuda', dtype=torch.float16).T
    else:
        a = torch.randn((M, K), device='cuda', dtype=torch.float16)
    if (TRANS_B):
        b = torch.randn((N, K), device='cuda', dtype=torch.float16).T
    else:
        b = torch.randn((K, N), device='cuda', dtype=torch.float16)

    c = torch.empty((M, N), device=a.device, dtype=torch.float32)
    if OUTPUT_F16:
        c = torch.empty((M, N), device=a.device, dtype=torch.float16)

    matmul_tma_load_store[(1, 1)](
        a_ptr=a, b_ptr=b, c_ptr=c,  #
        M=M, N=N, K=K,  #
        stride_am=a.stride(0), stride_ak=a.stride(1),  #
        stride_bk=b.stride(0), stride_bn=b.stride(1),  #
        stride_cm=c.stride(0), stride_cn=c.stride(1),  #
        BLOCK_M=M, BLOCK_N=N, BLOCK_K=K,  #
        num_warps=NUM_WARPS, num_ctas=NUM_CTAS,  #
        OUTPUT_F16=OUTPUT_F16)
    golden = torch.matmul(a, b)
    torch.set_printoptions(profile="full")

    result_gold['_CALL_SUCCESS_'] = torch.tensor([[1.0]])
    
    ################### save tri_out in result_gold ###################
    test_case_name = request.node.name
    sanitized_key_name = test_case_name.replace("::", "_").replace("[", "_").replace("]", "").replace("-", "_")
    result_gold[sanitized_key_name] = c.clone().detach().cpu()
    ###################################################################

    assert_close(c, golden, rtol=1e-2, atol=1e-3, check_dtype=False)


# --- Python wrapper for the kernel for benchmarking ---
def tma_gemm_triton_wrapper(a_tensor, b_tensor, c_buffer, 
                            M_dim, N_dim, K_dim, 
                            output_f16_flag, num_warps_launch): # num_ctas_launch removed as not used by grid
    grid = (1,) 
    matmul_tma_load_store[grid](
        a_ptr=a_tensor, b_ptr=b_tensor, c_ptr=c_buffer,
        M=M_dim, N=N_dim, K=K_dim,
        stride_am=a_tensor.stride(0), stride_ak=a_tensor.stride(1),
        stride_bk=b_tensor.stride(0), stride_bn=b_tensor.stride(1),
        stride_cm=c_buffer.stride(0), stride_cn=c_buffer.stride(1),
        BLOCK_M=M_dim, BLOCK_N=N_dim, BLOCK_K=K_dim, 
        OUTPUT_F16=output_f16_flag,
        num_warps=num_warps_launch
        # num_ctas=num_ctas_launch # Launch hint, not JIT param
    )
    return c_buffer

def calculate_tma_gemm_tflops(params: dict, ms: float) -> float:
    M, N, K = params['M'], params['N'], params['K']
    flops = 2 * M * N * K 
    tflops = flops / (ms / 1000) / 1e12
    return tflops

    
def calculate_tma_gemm_gbps(params: dict, ms: float) -> float:
    M, N, K = params['M'], params['N'], params['K']
    
    # Input dtype is fp16 as per test setup
    input_dtype_for_calc = torch.float16
    # Output C can be fp16 or fp32
    output_dtype_str = "fp16" if params['OUTPUT_F16'] else "fp32"
    
    output_torch_dtype_for_calc = torch.float16
    if output_dtype_str == 'fp32': output_torch_dtype_for_calc = torch.float32
    # No bf16 in this test's parametrize for output

    in_element_size = torch.tensor([], dtype=input_dtype_for_calc).element_size()
    out_element_size = torch.tensor([], dtype=output_torch_dtype_for_calc).element_size()
    
    bytes_a = M * K * in_element_size
    bytes_b = K * N * in_element_size 
    bytes_c_write = M * N * out_element_size
    total_bytes = bytes_a + bytes_b + bytes_c_write
    gbps = total_bytes / (ms / 1000) / 1e9
    return gbps

OP_NAME_FOR_BENCHMARK = "tma_store_gemm_triton_perf"

TMA_GEMM_PERF_PARAMS = [ 
    [64, 64, 16, 1, 4, False, True, False], [64, 64, 16, 1, 4, False, True, True],
    [128, 64, 32, 1, 4, False, True, False], [128, 64, 32, 1, 4, False, True, True],
    [64, 128, 32, 1, 4, False, True, False], [64, 128, 32, 1, 4, False, True, True],
    [128, 128, 64, 1, 4, False, True, False], [128, 128, 64, 1, 4, False, True, True],
    [64, 64, 128, 1, 4, False, False, True], 
    [64, 64, 128, 1, 4, False, False, False],
    [32, 32, 256, 1, 4, False, False, True],
]

@pytest.mark.parametrize('M,N,K,NUM_CTAS_param,NUM_WARPS_param,TRANS_A,TRANS_B,OUTPUT_F16_param', TMA_GEMM_PERF_PARAMS)
# @pytest.mark.skipif(torch.cuda.get_device_capability()[0] < 9, reason="Requires compute capability >= 9")
def test_performance(M, N, K, NUM_CTAS_param, NUM_WARPS_param, TRANS_A, TRANS_B, OUTPUT_F16_param, request):
    cap = torch.cuda.get_device_capability()
    if cap[0] < 9: 
        pytest.skip("Requires compute capability >= 9")

    # --- CORRECTED Shared memory skip logic ---
    input_torch_dtype_for_smem_check = torch.float16 # Inputs A and B are fp16 in this test
    element_size_bytes_for_smem_check = torch.tensor([], dtype=input_torch_dtype_for_smem_check).element_size()
    
    smem_elements_needed = (M * K) + (K * N) # For one dot A(M,K) @ B(K,N)
    if smem_elements_needed * element_size_bytes_for_smem_check > 65536: # 64KB limit
        pytest.skip(f"Skipping M{M}N{N}K{K} due to estimated shared memory for inputs: "
                    f"{smem_elements_needed * element_size_bytes_for_smem_check} > 65536")
    # --- End of corrected skip logic ---

    set_seed()
        
    # Input setup from original test, input dtype is fp16
    input_torch_dtype = torch.float16
    a_orig_shape = (K, M) if TRANS_A else (M, K)
    b_orig_shape = (N, K) if TRANS_B else (K, N) # If TRANS_B, B is (N,K), so B.T is (K,N) for matmul
    
    a = torch.randn(a_orig_shape, device='cuda', dtype=input_torch_dtype)
    if TRANS_A: a = a.T 
    
    b = torch.randn(b_orig_shape, device='cuda', dtype=input_torch_dtype)
    if TRANS_B: b = b.T 

    c_out_torch_dtype = torch.float16 if OUTPUT_F16_param else torch.float32
    c_buffer = torch.empty((M, N), device=a.device, dtype=c_out_torch_dtype)

    op_lambda = lambda: tma_gemm_triton_wrapper(
        a, b, c_buffer, M, N, K, 
        OUTPUT_F16_param, NUM_WARPS_param
    )

    bench_config = do_bench_config(warm_up=25, repetition=100)
    benchmarker = PytestBenchmarker(op_callable=op_lambda,
                                    op_name=OP_NAME_FOR_BENCHMARK,
                                    config=bench_config)
    current_params_for_logs_and_calc = {
        "M": M, "N": N, "K": K, 
        "NUM_CTAS": NUM_CTAS_param, "NUM_WARPS": NUM_WARPS_param,
        "TRANS_A": TRANS_A, "TRANS_B": TRANS_B, 
        "OUTPUT_F16": OUTPUT_F16_param, # This is for the calculator
        "input_dtype_str": "fp16" # For the calculator
    }
    
    perf_result = benchmarker.run_benchmark(current_params_dict=current_params_for_logs_and_calc,
                                            gbps_calculator=calculate_tma_gemm_gbps,
                                            tflops_calculator=calculate_tma_gemm_tflops)

######################################## HELPERS for Eval ########################################     
# --- Pytest hook to save the dictionary at the end of the session ---  
def test_save_results():  
    """  
    Called after whole test run finished, right before returning the exit status to the system.  
    """
    print('Inside session finish...')
    if "_CALL_SUCCESS_" not in result_gold:
        result_gold['_CALL_SUCCESS_'] = torch.tensor([[0.0]])
    OUTPUT_FILENAME = __file__.replace('.','_') + '.pt'
    print(f"\nSaving all y_triton results to {OUTPUT_FILENAME}...")  
    # Ensure the directory for the output file exists if it's in a subdirectory  
    output_dir = os.path.dirname(OUTPUT_FILENAME)  
    if output_dir and not os.path.exists(output_dir):  
        os.makedirs(output_dir, exist_ok=True)  
    torch.save(result_gold, OUTPUT_FILENAME)       
    print(f"Successfully saved {len(result_gold)} y_triton tensors to {OUTPUT_FILENAME}.")  


def test_save_performance_results():
    """
    Called after the test_performance function finishes.
    This is a separate hook to ensure performance results are saved.
    """
    print('\nPytest session finishing... Saving benchmark results...')

    output_directory = os.path.join(os.path.dirname(__file__), "perf")  # Save in a "perf" subdirectory next to the test file
    os.makedirs(output_directory, exist_ok=True)
    
    save_all_benchmark_results(output_directory)
    print(f"All benchmark results attempted to save to: {output_directory}")

######################################## HELPERS for Eval ########################################


import sys
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from TritonBench_v1.triton_conv2d_fwd import conv2d_forward
from performance_utils import Performance_Metrics, do_bench_config

import torch
import triton
import triton.language as tl

class performance_metrics(Performance_Metrics):
    def __init__(self, dtype=None, is_backward=False, **kwargs):
        super().__init__('triton_conv2d_fwd', dtype=dtype, is_backward=is_backward, **kwargs)
        
    def get_input_tensors(self):
        self.input_tensors = []
        for i in range(2, 10):  # Example sizes, adjust as needed
            batch_size = 2 ** i
            in_channels = 3
            out_channels = 64
            height = 32
            width = 32
            kernel_size = 3
            stride = 1
            padding = 1
            input_tensor = torch.rand((batch_size, in_channels, height, width), dtype=torch.float32)
            weight_tensor = torch.rand((out_channels, in_channels, kernel_size, kernel_size), dtype=torch.float32)
            self.input_tensors.append((input_tensor, weight_tensor, kernel_size, kernel_size, stride, stride, padding, padding, 1))

    def to_cuda(self, input_tensor):
        input_tensor_cuda = input_tensor[0].cuda()
        weight_tensor_cuda = input_tensor[1].cuda()
        return (input_tensor_cuda, weight_tensor_cuda) + input_tensor[2:]

    def call_op(self, input_tensor):
        return conv2d_forward(*input_tensor)

    def get_gbps(self, input_tensor, runtime):
        input_tensor_size = input_tensor[0].numel() * input_tensor[0].element_size()
        weight_tensor_size = input_tensor[1].numel() * input_tensor[1].element_size()
        output_tensor_size = self.call_op(input_tensor).numel() * input_tensor[0].element_size()
        total_bytes = input_tensor_size + weight_tensor_size + output_tensor_size
        GBPS = total_bytes / (runtime / 1000) / 1e9
        return GBPS
    
    def get_tflops(self, input_tensor, runtime):
        batch_dim, in_feat_dim, in_height, in_width = input_tensor[0].shape
        out_feat_dim, _, kernel_height, kernel_width = input_tensor[1].shape
        out_height = (in_height + 2 * input_tensor[6] - kernel_height) // input_tensor[4] + 1
        out_width = (in_width + 2 * input_tensor[7] - kernel_width) // input_tensor[5] + 1
        FLOPS = 2 * batch_dim * out_feat_dim * out_height * out_width * in_feat_dim * kernel_height * kernel_width
        TFLOPS = FLOPS / (runtime / 1000) / 1e12
        return TFLOPS

if __name__ == '__main__':
    op_perf = performance_metrics()
    op_perf.get_input_tensors()
    op_perf.get_do_bench_config(warmup=100, rep=1000)
    op_perf.run_benchmark()



import sys
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from TritonBench_v1.bgmv_expand_slice import _bgmv_expand_slice
from performance_utils import Performance_Metrics, do_bench_config

import torch
import triton
import triton.language as tl

class performance_metrics(Performance_Metrics):
    def __init__(self, dtype=None, is_backward=False, **kwargs):
        super().__init__('bgmv_expand_slice', dtype=dtype, is_backward=is_backward, **kwargs)
        
    def get_input_tensors(self):
        self.input_tensors = []
        for i in range(2, 12):  # Adjust the range as needed for your tests
            size = 2 ** i
            N = size
            K = size
            inputs = torch.rand((1, K), dtype=torch.float32)
            lora_b_weights = torch.rand((1, N, K), dtype=torch.float16)
            output_tensor = torch.zeros((1, N), dtype=torch.float32)
            lora_indices_tensor = torch.tensor([0], dtype=torch.int32)
            slice_offset = 0
            slice_size = N
            self.input_tensors.append((inputs, lora_b_weights, output_tensor, lora_indices_tensor, slice_offset, slice_size))

    def to_cuda(self, input_tensor):
        inputs, lora_b_weights, output_tensor, lora_indices_tensor, slice_offset, slice_size = input_tensor
        return (inputs.cuda(), lora_b_weights.cuda(), output_tensor.cuda(), lora_indices_tensor.cuda(), slice_offset, slice_size)

    def call_op(self, input_tensor):
        inputs, lora_b_weights, output_tensor, lora_indices_tensor, slice_offset, slice_size = input_tensor
        _bgmv_expand_slice(inputs, lora_b_weights, output_tensor, lora_indices_tensor, slice_offset, slice_size)
        return output_tensor

    def get_gbps(self, input_tensor, runtime):
        inputs, lora_b_weights, output_tensor, lora_indices_tensor, slice_offset, slice_size = input_tensor
        total_bytes = inputs.numel() * inputs.element_size() + \
                      lora_b_weights.numel() * lora_b_weights.element_size() + \
                      output_tensor.numel() * output_tensor.element_size()
        GBPS = total_bytes / (runtime / 1000) / 1e9
        return GBPS
    
    def get_tflops(self, input_tensor, runtime):
        inputs, lora_b_weights, output_tensor, lora_indices_tensor, slice_offset, slice_size = input_tensor
        N, K = lora_b_weights.shape[-2:]
        FLOPS = 2 * N * K  # Assuming a matrix-vector multiplication
        TFLOPS = FLOPS / (runtime / 1000) / 1e12
        return TFLOPS

if __name__ == '__main__':
    op_perf = performance_metrics()
    op_perf.get_input_tensors()
    op_perf.get_do_bench_config(warmup=100, rep=1000)
    op_perf.run_benchmark()



import sys
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from TritonBench_v1.batched_vecmat_mult import batched_vecmat
from performance_utils import Performance_Metrics, do_bench_config

import torch
import triton
import triton.language as tl

class performance_metrics(Performance_Metrics):
    def __init__(self, dtype=None, is_backward=False, **kwargs):
        super().__init__('batched_vecmat_mult', dtype=dtype, is_backward=is_backward, **kwargs)
        
    def get_input_tensors(self):
        self.input_tensors = []
        for i in range(2, 16):  # Adjust the range as needed for your testing
            M = 16 * i
            N = 16 * i
            K = 16 * i
            block_m = 16  # Example block size, adjust as needed
            block_n = 16
            block_k = 16
            input_tensor = (M, N, K, block_m, block_n, block_k)
            self.input_tensors.append(input_tensor)

    def to_cuda(self, input_tensor):
        # No need to transfer to CUDA as the batched_vecmat function already initializes tensors on CUDA
        return input_tensor

    def call_op(self, input_tensor):
        M, N, K, block_m, block_n, block_k = input_tensor
        return batched_vecmat(M, N, K, block_m, block_n, block_k)

    def get_gbps(self, input_tensor, runtime):
        M, N, K, block_m, block_n, block_k = input_tensor
        # Calculate the total bytes processed
        total_bytes = (M * K + M * N * K + M * N) * 4  # 4 bytes for float32
        GBPS = total_bytes / (runtime / 1000) / 1e9
        return GBPS
    
    def get_tflops(self, input_tensor, runtime):
        M, N, K, block_m, block_n, block_k = input_tensor
        # Calculate the total floating-point operations
        FLOPS = 2 * M * N * K
        TFLOPS = FLOPS / (runtime / 1000) / 1e12
        return TFLOPS

if __name__ == '__main__':
    op_perf = performance_metrics()
    op_perf.get_input_tensors()
    op_perf.get_do_bench_config(warmup=100, rep=1000)
    op_perf.run_benchmark()



# Modifications Copyright(C)[2025] Advanced Micro Devices, Inc. All rights reserved.
# https://github.com/thunlp/TritonBench - Apache License 2.0
import torch
import triton
import triton.language as tl


def _matmul_launch_metadata(grid, kernel, args):
    ret = {}
    M, N, K = args["M"], args["N"], args["K"]
    ret["name"] = f"{kernel.name} [M={M}, N={N}, K={K}]"
    if "c_ptr" in args:
        bytes_per_elem = args["c_ptr"].element_size()
    else:
        bytes_per_elem = 1 if args["FP8_OUTPUT"] else 2
    ret[f"flops{bytes_per_elem * 8}"] = 2. * M * N * K
    ret["bytes"] = bytes_per_elem * (M * K + N * K + M * N)
    return ret


@triton.jit(launch_metadata=_matmul_launch_metadata)
def matmul_kernel_persistent(a_ptr, b_ptr, c_ptr,  #
                             M, N, K,  #
                             stride_am, stride_ak,  #
                             stride_bk, stride_bn,  #
                             stride_cm, stride_cn,  #
                             BLOCK_SIZE_M: tl.constexpr,  #
                             BLOCK_SIZE_N: tl.constexpr,  #
                             BLOCK_SIZE_K: tl.constexpr,  #
                             GROUP_SIZE_M: tl.constexpr,  #
                             NUM_SMS: tl.constexpr,  #
                             ):
    start_pid = tl.program_id(axis=0)
    num_pid_m = tl.cdiv(M, BLOCK_SIZE_M)
    num_pid_n = tl.cdiv(N, BLOCK_SIZE_N)
    k_tiles = tl.cdiv(K, BLOCK_SIZE_K)
    num_tiles = num_pid_m * num_pid_n

    tiles_per_SM = num_tiles // NUM_SMS
    if start_pid < num_tiles % NUM_SMS:
        tiles_per_SM += 1

    tile_id = start_pid - NUM_SMS
    ki = -1

    offs_k_for_mask = tl.arange(0, BLOCK_SIZE_K)

    num_pid_in_group = GROUP_SIZE_M * num_pid_n

    pid_m = 0
    pid_n = 0
    offs_am = tl.arange(0, BLOCK_SIZE_M)
    offs_bn = tl.arange(0, BLOCK_SIZE_N)

    accumulator = tl.zeros((BLOCK_SIZE_M, BLOCK_SIZE_N), dtype=tl.float32)

    for _ in range(0, k_tiles * tiles_per_SM):
        ki = tl.where(ki == k_tiles - 1, 0, ki + 1)
        if ki == 0:
            tile_id += NUM_SMS
            group_id = tile_id // num_pid_in_group
            first_pid_m = group_id * GROUP_SIZE_M
            group_size_m = min(num_pid_m - first_pid_m, GROUP_SIZE_M)
            pid_m = first_pid_m + (tile_id % group_size_m)
            pid_n = (tile_id % num_pid_in_group) // group_size_m

            start_m = pid_m * BLOCK_SIZE_M
            start_n = pid_n * BLOCK_SIZE_N
            offs_am = start_m + tl.arange(0, BLOCK_SIZE_M)
            offs_bn = start_n + tl.arange(0, BLOCK_SIZE_N)
            offs_am = tl.where(offs_am < M, offs_am, 0)
            offs_bn = tl.where(offs_bn < N, offs_bn, 0)
            offs_am = tl.max_contiguous(tl.multiple_of(offs_am, BLOCK_SIZE_M), BLOCK_SIZE_M)
            offs_bn = tl.max_contiguous(tl.multiple_of(offs_bn, BLOCK_SIZE_N), BLOCK_SIZE_N)
        offs_k = ki * BLOCK_SIZE_K + tl.arange(0, BLOCK_SIZE_K)
        a_ptrs = a_ptr + (offs_am[:, None] * stride_am + offs_k[None, :] * stride_ak)
        b_ptrs = b_ptr + (offs_k[:, None] * stride_bk + offs_bn[None, :] * stride_bn)

        a = tl.load(a_ptrs, mask=offs_k_for_mask[None, :] < K - ki * BLOCK_SIZE_K, other=0.0)
        b = tl.load(b_ptrs, mask=offs_k_for_mask[:, None] < K - ki * BLOCK_SIZE_K, other=0.0)
        accumulator = tl.dot(a, b, accumulator)

        if ki == k_tiles - 1:
            offs_cm = pid_m * BLOCK_SIZE_M + tl.arange(0, BLOCK_SIZE_M)
            offs_cn = pid_n * BLOCK_SIZE_N + tl.arange(0, BLOCK_SIZE_N)
            c_ptrs = c_ptr + stride_cm * offs_cm[:, None] + stride_cn * offs_cn[None, :]
            c_mask = (offs_cm[:, None] < M) & (offs_cn[None, :] < N)
            if (c_ptr.dtype.element_ty == tl.float8e4nv):
                c = accumulator.to(tl.float8e4nv)
            else:
                c = accumulator.to(tl.float16)
            tl.store(c_ptrs, c, mask=c_mask)
            accumulator = tl.zeros((BLOCK_SIZE_M, BLOCK_SIZE_N), dtype=tl.float32)


def matmul_persistent(a, b):
    configs = {
        torch.float8_e4m3fn: {
            "BLOCK_SIZE_M": 128, "BLOCK_SIZE_N": 256, "BLOCK_SIZE_K": 128, "GROUP_SIZE_M": 8, "num_stages": 2,
            "num_warps": 8
        }, torch.float16: {
            "BLOCK_SIZE_M": 128, "BLOCK_SIZE_N": 256, "BLOCK_SIZE_K": 64, "GROUP_SIZE_M": 8, "num_stages": 1,
            "num_warps": 8
        }
    }
    # Check constraints.
    assert a.shape[1] == b.shape[0], "Incompatible dimensions"
    assert a.dtype == b.dtype, "Incompatible dtypes"
    NUM_SMS = torch.cuda.get_device_properties("cuda").multi_processor_count
    M, K = a.shape
    K, N = b.shape
    dtype = a.dtype
    # Allocates output.
    c = torch.empty((M, N), device=a.device, dtype=dtype)
    # 1D launch kernel where each block gets its own program.
    grid = lambda META: (min(NUM_SMS, triton.cdiv(M, META["BLOCK_SIZE_M"]) * triton.cdiv(N, META["BLOCK_SIZE_N"])), )
    matmul_kernel_persistent[grid](
        a, b, c,  #
        M, N, K,  #
        a.stride(0), a.stride(1),  #
        b.stride(0), b.stride(1),  #
        c.stride(0), c.stride(1),  #
        BLOCK_SIZE_M=configs[dtype]["BLOCK_SIZE_M"],  #
        BLOCK_SIZE_N=configs[dtype]["BLOCK_SIZE_N"],  #
        BLOCK_SIZE_K=configs[dtype]["BLOCK_SIZE_K"],  #
        GROUP_SIZE_M=configs[dtype]["GROUP_SIZE_M"],  #
        NUM_SMS=NUM_SMS,  #
        num_stages=configs[dtype]["num_stages"],  #
        num_warps=configs[dtype]["num_warps"],  #
    )
    return c


##################################################################################################################################################


import torch

# Test for matmul_persistent
def test_matmul_persistent():
    M, K, N = 256, 128, 256
    results = {}
    
    # Test case 1
    a = torch.randn((M, K), dtype=torch.float16, device='cuda')
    b = torch.randn((K, N), dtype=torch.float16, device='cuda')
    c = matmul_persistent(a, b)
    results['test_case_1'] = c

    return results

# Run all tests
result_gold = test_matmul_persistent()
# test_matmul_persistent()



import sys
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from TritonBench_v1.vector_addition_custom import custom_add
from performance_utils import Performance_Metrics, do_bench_config

import torch
import triton
import triton.language as tl

class performance_metrics(Performance_Metrics):
    def __init__(self, dtype=None, is_backward=False, **kwargs):
        super().__init__('vector_addition_custom', dtype=dtype, is_backward=is_backward, **kwargs)
        
    def get_input_tensors(self):
        self.input_tensors = []
        for i in range(12, 28):
            size = 2 ** i
            input_tensor = (torch.rand(size, dtype=torch.float32), torch.rand(size, dtype=torch.float32))
            self.input_tensors.append(input_tensor)

    def to_cuda(self, input_tensor):
        return (input_tensor[0].cuda(), input_tensor[1].cuda())

    def call_op(self, input_tensor):
        return custom_add(input_tensor[0], input_tensor[1])

    def get_gbps(self, input_tensor, runtime):
        x = input_tensor[0]
        total_bytes = 3 * x.numel() * x.element_size()
        GBPS = total_bytes / (runtime / 1000) / 1e9
        return GBPS
    
    def get_tflops(self, input_tensor, runtime):
        x = input_tensor[0]
        FLOPS = 2 * float(x.size()[0])
        TFLOPS = FLOPS / (runtime / 1000) / 1e12
        return TFLOPS

if __name__ == '__main__':
    op_perf = performance_metrics()
    op_perf.get_input_tensors()
    op_perf.get_do_bench_config(warmup=100, rep=1000)
    op_perf.run_benchmark()



# Modifications Copyright(C)[2025] Advanced Micro Devices, Inc. All rights reserved.
# https://github.com/thunlp/TritonBench - Apache License 2.0
from typing import Optional, Union
import torch
import triton
import triton.language as tl

@triton.jit
def rotary_kernel(
    OUT, X, COS, SIN, CU_SEQLENS, SEQLEN_OFFSETS, seqlen, nheads, rotary_dim, seqlen_ro,
    CACHE_KEY_SEQLEN, stride_out_batch, stride_out_seqlen, stride_out_nheads,
    stride_out_headdim, stride_x_batch, stride_x_seqlen, stride_x_nheads,
    stride_x_headdim, BLOCK_K: tl.constexpr, IS_SEQLEN_OFFSETS_TENSOR: tl.constexpr,
    IS_VARLEN: tl.constexpr, INTERLEAVED: tl.constexpr, CONJUGATE: tl.constexpr, BLOCK_M: tl.constexpr,
):
    pid_m = tl.program_id(axis=0)
    pid_batch = tl.program_id(axis=1)
    pid_head = tl.program_id(axis=2)
    rotary_dim_half = rotary_dim // 2

    if not IS_VARLEN:
        X = X + pid_batch * stride_x_batch + pid_head * stride_x_nheads
        OUT = OUT + pid_batch * stride_out_batch + pid_head * stride_out_nheads
    else:
        start_idx = tl.load(CU_SEQLENS + pid_batch)
        seqlen = tl.load(CU_SEQLENS + pid_batch + 1) - start_idx
        X = X + start_idx * stride_x_seqlen + pid_head * stride_x_nheads
        OUT = OUT + start_idx * stride_out_seqlen + pid_head * stride_out_nheads

    if pid_m * BLOCK_M >= seqlen:
        return
    rm = pid_m * BLOCK_M + tl.arange(0, BLOCK_M)
    if not IS_SEQLEN_OFFSETS_TENSOR:
        rm_cs = rm + SEQLEN_OFFSETS
    else:
        rm_cs = rm + tl.load(SEQLEN_OFFSETS + pid_batch)
    rk = tl.arange(0, BLOCK_K)
    rk_half = tl.arange(0, BLOCK_K // 2)

    if not INTERLEAVED:
        X = X + (rm[:, None] * stride_x_seqlen + rk_half[None, :] * stride_x_headdim)
        COS = COS + (rm_cs[:, None] * rotary_dim_half + rk_half[None, :])
        SIN = SIN + (rm_cs[:, None] * rotary_dim_half + rk_half[None, :])
        cos = tl.load(COS, mask=(rm_cs[:, None] < seqlen_ro) & (rk_half[None, :] < rotary_dim_half), other=1.0).to(tl.float32)
        sin = tl.load(SIN, mask=(rm_cs[:, None] < seqlen_ro) & (rk_half[None, :] < rotary_dim_half), other=0.0).to(tl.float32)
        x0 = tl.load(X, mask=(rm[:, None] < seqlen) & (rk_half[None, :] < rotary_dim_half), other=0.0).to(tl.float32)
        x1 = tl.load(X + rotary_dim_half * stride_x_headdim, mask=(rm[:, None] < seqlen) & (rk_half[None, :] < rotary_dim_half), other=0.0).to(tl.float32)
        if CONJUGATE:
            sin = -sin
        o0 = x0 * cos - x1 * sin
        o1 = x0 * sin + x1 * cos
        OUT = OUT + (rm[:, None] * stride_out_seqlen + rk_half[None, :] * stride_out_headdim)
        tl.store(OUT, o0, mask=(rm[:, None] < seqlen) & (rk_half[None, :] < rotary_dim_half))
        tl.store(OUT + rotary_dim_half * stride_out_headdim, o1, mask=(rm[:, None] < seqlen) & (rk_half[None, :] < rotary_dim_half))
    else:
        rk_swap = rk + ((rk + 1) % 2) * 2 - 1
        rk_repeat = tl.arange(0, BLOCK_K) // 2
        X0 = X + (rm[:, None] * stride_x_seqlen + rk[None, :] * stride_x_headdim)
        X1 = X + (rm[:, None] * stride_x_seqlen + rk_swap[None, :] * stride_x_headdim)
        COS = COS + (rm_cs[:, None] * rotary_dim_half + rk_repeat[None, :])
        SIN = SIN + (rm_cs[:, None] * rotary_dim_half + rk_repeat[None, :])
        cos = tl.load(COS, mask=(rm_cs[:, None] < seqlen_ro) & (rk_repeat[None, :] < rotary_dim_half), other=1.0).to(tl.float32)
        sin = tl.load(SIN, mask=(rm_cs[:, None] < seqlen_ro) & (rk_repeat[None, :] < rotary_dim_half), other=0.0).to(tl.float32)
        x0 = tl.load(X0, mask=(rm[:, None] < seqlen) & (rk[None, :] < rotary_dim), other=0.0).to(tl.float32)
        x1 = tl.load(X1, mask=(rm[:, None] < seqlen) & (rk_swap[None, :] < rotary_dim), other=0.0).to(tl.float32)
        if CONJUGATE:
            sin = -sin
        x0_cos = x0 * cos
        x1_sin = x1 * sin
        out = tl.where(rk[None, :] % 2 == 0, x0_cos - x1_sin, x0_cos + x1_sin)
        OUT = OUT + (rm[:, None] * stride_out_seqlen + rk[None, :] * stride_out_headdim)
        tl.store(OUT, out, mask=(rm[:, None] < seqlen) & (rk[None, :] < rotary_dim))

def apply_rotary(
    x: torch.Tensor, cos: torch.Tensor, sin: torch.Tensor, seqlen_offsets: Union[int, torch.Tensor] = 0,
    cu_seqlens: Optional[torch.Tensor] = None, max_seqlen: Optional[int] = None,
    interleaved=False, inplace=False, conjugate=False
) -> torch.Tensor:
    is_varlen = cu_seqlens is not None
    if not is_varlen:
        batch, seqlen, nheads, headdim = x.shape
    else:
        total_seqlen, nheads, headdim = x.shape
        batch = cu_seqlens.shape[0] - 1
        seqlen = max_seqlen
    seqlen_ro, rotary_dim = cos.shape
    rotary_dim *= 2

    cos, sin = cos.contiguous(), sin.contiguous()
    if isinstance(seqlen_offsets, torch.Tensor):
        seqlen_offsets = seqlen_offsets.contiguous()
    else:
        seqlen_offsets += seqlen

    output = torch.empty_like(x) if not inplace else x
    if rotary_dim < headdim and not inplace:
        output[..., rotary_dim:].copy_(x[..., rotary_dim:])

    BLOCK_K = 32 if rotary_dim <= 32 else (64 if rotary_dim <= 64 else (128 if rotary_dim <= 128 else 256))
    grid = lambda META: (triton.cdiv(seqlen, META["BLOCK_M"]), batch, nheads)
    BLOCK_M = 4 if interleaved else (8 if rotary_dim <= 64 else 4)

    with torch.cuda.device(x.device.index):
        rotary_kernel[grid](
            output, x, cos, sin, cu_seqlens, seqlen_offsets, seqlen, nheads, rotary_dim,
            seqlen_ro, seqlen // 128, output.stride(0) if not is_varlen else 0, output.stride(-3),
            output.stride(-2), output.stride(-1), x.stride(0) if not is_varlen else 0,
            x.stride(-3), x.stride(-2), x.stride(-1), BLOCK_K,
            isinstance(seqlen_offsets, torch.Tensor), is_varlen, interleaved, conjugate, BLOCK_M
        )
    return output




##################################################################################################################################################


import torch

def test_apply_rotary():
    results = {}
    
    # Test case 1: Basic test with fixed sequence length and no interleaving
    batch, seqlen, nheads, headdim = 2, 128, 4, 64
    rotary_dim = 32
    x = torch.randn(batch, seqlen, nheads, headdim, device='cuda')
    cos = torch.randn(seqlen, rotary_dim // 2, device='cuda')
    sin = torch.randn(seqlen, rotary_dim // 2, device='cuda')
    output = apply_rotary(x, cos, sin)
    results['test_case_1'] = output.shape

    # Test case 2: Variable length sequences with interleaving
    total_seqlen, nheads, headdim = 256, 4, 64
    batch = 3
    cu_seqlens = torch.tensor([0, 100, 200, 256], device='cuda')
    max_seqlen = 128
    rotary_dim = 32
    x = torch.randn(total_seqlen, nheads, headdim, device='cuda')
    cos = torch.randn(max_seqlen, rotary_dim // 2, device='cuda')
    sin = torch.randn(max_seqlen, rotary_dim // 2, device='cuda')
    output = apply_rotary(x, cos, sin, cu_seqlens=cu_seqlens, max_seqlen=max_seqlen, interleaved=True)
    results['test_case_2'] = output.shape

    # Test case 3: Conjugate flag enabled
    batch, seqlen, nheads, headdim = 2, 128, 4, 64
    rotary_dim = 32
    x = torch.randn(batch, seqlen, nheads, headdim, device='cuda')
    cos = torch.randn(seqlen, rotary_dim // 2, device='cuda')
    sin = torch.randn(seqlen, rotary_dim // 2, device='cuda')
    output = apply_rotary(x, cos, sin, conjugate=True)
    results['test_case_3'] = output.shape

    # Test case 4: Inplace operation
    batch, seqlen, nheads, headdim = 2, 128, 4, 64
    rotary_dim = 32
    x = torch.randn(batch, seqlen, nheads, headdim, device='cuda')
    cos = torch.randn(seqlen, rotary_dim // 2, device='cuda')
    sin = torch.randn(seqlen, rotary_dim // 2, device='cuda')
    output = apply_rotary(x, cos, sin, inplace=True)
    results['test_case_4'] = output.shape

    return results

result_gold = test_apply_rotary()



import sys
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from TritonBench_v1.embedding_triton_kernel import embedding
from performance_utils import Performance_Metrics, do_bench_config

import torch
import triton
import triton.language as tl

class performance_metrics(Performance_Metrics):
    def __init__(self, dtype=None, is_backward=False, **kwargs):
        super().__init__('embedding_triton_kernel', dtype=dtype, is_backward=is_backward, **kwargs)
        
    def get_input_tensors(self):
        self.input_tensors = []
        vocab_size = 10000  # Example vocabulary size
        hidden_size = 768   # Example hidden size
        for i in range(2, 18):  # Example range for input sizes
            seq_length = 2 ** i
            input_ids = torch.randint(0, vocab_size, (seq_length,), dtype=torch.int32)
            weight = torch.rand(vocab_size, hidden_size, dtype=torch.float16)
            out = torch.zeros(seq_length, hidden_size, dtype=torch.float16)
            vob_start_id = 0
            vob_end_id = vocab_size
            self.input_tensors.append((input_ids, weight, vob_start_id, vob_end_id, out))

    def to_cuda(self, input_tensor):
        input_ids, weight, vob_start_id, vob_end_id, out = input_tensor
        return (input_ids.cuda(), weight.cuda(), vob_start_id, vob_end_id, out.cuda())

    def call_op(self, input_tensor):
        input_ids, weight, vob_start_id, vob_end_id, out = input_tensor
        embedding(input_ids, weight, vob_start_id, vob_end_id, out)
        return out

    def get_gbps(self, input_tensor, runtime):
        input_ids, weight, _, _, _ = input_tensor
        total_bytes = input_ids.numel() * input_ids.element_size() + weight.numel() * weight.element_size()
        GBPS = total_bytes / (runtime / 1000) / 1e9
        return GBPS
    
    def get_tflops(self, input_tensor, runtime):
        input_ids, weight, _, _, _ = input_tensor
        FLOPS = 2 * input_ids.numel() * weight.shape[1]  # Assuming 2 FLOPS per element (load and store)
        TFLOPS = FLOPS / (runtime / 1000) / 1e12
        return TFLOPS

if __name__ == '__main__':
    op_perf = performance_metrics()
    op_perf.get_input_tensors()
    op_perf.get_do_bench_config(warmup=100, rep=1000)
    op_perf.run_benchmark()



import sys
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from TritonBench_v1.token_attn_llama2 import token_att_fwd
from performance_utils import Performance_Metrics, do_bench_config

import torch
import triton
import triton.language as tl

class performance_metrics(Performance_Metrics):
    def __init__(self, dtype=None, is_backward=False, **kwargs):
        super().__init__('token_attn_llama2', dtype=dtype, is_backward=is_backward, **kwargs)
        
    def get_input_tensors(self):
        self.input_tensors = []
        for i in range(5, 16):  # Example sizes, adjust as needed
            batch_size = 2 ** i
            head_num = 8
            seq_len = 128
            d_model = 64
            max_input_len = seq_len

            q = torch.rand((batch_size, head_num, d_model), dtype=torch.float16)
            k = torch.rand((batch_size, head_num, d_model), dtype=torch.float16)
            att_out = torch.zeros((batch_size, head_num, seq_len), dtype=torch.float16)
            B_Loc = torch.randint(0, seq_len, (batch_size, seq_len), dtype=torch.int32)
            B_Start_Loc = torch.zeros(batch_size, dtype=torch.int32)
            B_Seqlen = torch.full((batch_size,), seq_len, dtype=torch.int32)

            self.input_tensors.append((q, k, att_out, B_Loc, B_Start_Loc, B_Seqlen, max_input_len))

    def to_cuda(self, input_tensor):
        q, k, att_out, B_Loc, B_Start_Loc, B_Seqlen, max_input_len = input_tensor
        return (q.cuda(), k.cuda(), att_out.cuda(), B_Loc.cuda(), B_Start_Loc.cuda(), B_Seqlen.cuda(), max_input_len)

    def call_op(self, input_tensor):
        q, k, att_out, B_Loc, B_Start_Loc, B_Seqlen, max_input_len = input_tensor
        return token_att_fwd(q, k, att_out, B_Loc, B_Start_Loc, B_Seqlen, max_input_len)

    def get_gbps(self, input_tensor, runtime):
        q, k, att_out, B_Loc, B_Start_Loc, B_Seqlen, max_input_len = input_tensor
        total_bytes = (q.numel() + k.numel() + att_out.numel()) * q.element_size() + B_Loc.numel() * B_Loc.element_size()
        GBPS = total_bytes / (runtime / 1000) / 1e9
        return GBPS
    
    def get_tflops(self, input_tensor, runtime):
        q, k, att_out, B_Loc, B_Start_Loc, B_Seqlen, max_input_len = input_tensor
        # Assuming the main computation is the dot product in the attention mechanism
        FLOPS = 2 * q.size(0) * q.size(1) * q.size(2) * max_input_len
        TFLOPS = FLOPS / (runtime / 1000) / 1e12
        return TFLOPS

if __name__ == '__main__':
    op_perf = performance_metrics()
    op_perf.get_input_tensors()
    op_perf.get_do_bench_config(warmup=100, rep=1000)
    op_perf.run_benchmark()



# Copyright(C) [2025] Advanced Micro Devices, Inc. All rights reserved.
######################################## Imports ########################################
import pytest
import torch
import triton
import triton.language as tl
# numpy is not strictly needed here if output is directly compared
######################################## Imports ########################################

@triton.jit
def swizzle2d_kernel(output, size_i, size_j, size_g):
    for i in tl.range(0, size_i, 1):
        for j in tl.range(0, size_j, 1):
            new_i, new_j = tl.swizzle2d(i, j, size_i, size_j, size_g)
            tl.store(output + new_i * size_j + new_j, i * size_j + j)

##################################################################################################################################################  

import numpy as np
import random
import torch 
import os
import pytest
from numpy.random import RandomState
from geak_eval.perf.ROCm.performance_utils_pytest import PytestBenchmarker, do_bench_config, save_all_benchmark_results
from typing import Dict

result_gold = {}

######################################## HELPERS for Eval ######################################## 
def set_seed(seed: int = 42) -> None:
    """
    Set the random seed for reproducibility across multiple libraries and configure PyTorch for deterministic behavior.

    Args:
        seed (int): The seed value to set. Default is 42.
    """
    # Set seed for Python's built-in random module
    random.seed(seed)
    # Set seed for NumPy
    np.random.seed(seed)
    # Set seed for PyTorch on CPU
    torch.manual_seed(seed)
    # Set seed for PyTorch on all GPUs (if available)
    if torch.cuda.is_available():
        torch.cuda.manual_seed(seed)
        torch.cuda.manual_seed_all(seed)
    # Ensure deterministic behavior in PyTorch
    torch.backends.cudnn.deterministic = True
    torch.backends.cudnn.benchmark = False
    # Set environment variable for hash-based operations
    os.environ['PYTHONHASHSEED'] = str(seed)

######################################## HELPERS for Eval ######################################## 





@pytest.mark.interpreter
@pytest.mark.parametrize("size_i, size_j, size_g", [[5, 7, 3]])
def test_swizzle2d(size_i, size_j, size_g, request, device='cuda'):
    # Output tensor to store results, initialized to a value like -1 to see what's written

    set_seed()
    
    output = torch.zeros(size_i, size_j).to(device)
    swizzle2d_kernel[(1, )](output, size_i, size_j, size_g)
    expected_order = torch.tensor([[0, 3, 6, 9, 12, 15, 18], [1, 4, 7, 10, 13, 16, 19], [2, 5, 8, 11, 14, 17, 20],
                                   [21, 23, 25, 27, 29, 31, 33], [22, 24, 26, 28, 30, 32, 34]]).to(device)
    
    result_gold['_CALL_SUCCESS_'] = torch.tensor([[1.0]])

    ################### save tri_out in result_gold ###################
    test_case_name = request.node.name
    sanitized_key_name = test_case_name.replace("::", "_").replace("[", "_").replace("]", "").replace("-", "_")
    result_gold[sanitized_key_name] = output.clone().detach().cpu()
    ################################################################### 

    assert (output == expected_order).all(), (output, expected_order)


# --- Python wrapper for the kernel for benchmarking ---
def swizzle2d_triton_wrapper(output_buffer, size_i_k, size_j_k, size_g_k, num_warps_launch):
    grid = (1,) # Kernel is sequential, not tiled by program_id
    swizzle2d_kernel[grid](
        output_buffer, 
        size_i_k, size_j_k, size_g_k, # Runtime args to kernel
        num_warps=num_warps_launch # Launch hint
    )
    return output_buffer

# --- Define TFLOPS and GB/s calculators ---
def calculate_swizzle2d_tflops(params: dict, ms: float) -> float:
    # tl.swizzle2d involves bitwise ops (shifts, xors, ands) and some arithmetic.
    # A rough estimate might be ~10-20 simple ops per (i,j) pair.
    # Number of pairs = size_i * size_j
    size_i = params['size_i_kernel']
    size_j = params['size_j_kernel']
    ops_per_swizzle = 15 # Rough estimate
    total_ops = size_i * size_j * ops_per_swizzle
    tflops = total_ops / (ms / 1000) / 1e12 # These are integer ops, not float ops
    return tflops # Reporting "integer TOPS" effectively

def calculate_swizzle2d_gbps(params: dict, ms: float) -> float:
    size_i = params['size_i_kernel']
    size_j = params['size_j_kernel']
    dtype_str = params.get('output_dtype_str', 'int32') # Kernel stores integers

    current_dtype = torch.int32 # Default
    if dtype_str == 'int64': current_dtype = torch.int64
    # Add other int types if parametrized
    element_size = torch.tensor([], dtype=current_dtype).element_size()

    # Writes size_i * size_j elements to output. No global memory reads for input data.
    elements_written = size_i * size_j
    total_bytes = elements_written * element_size
    gbps = total_bytes / (ms / 1000) / 1e9
    return gbps

OP_NAME_FOR_BENCHMARK = "triton_swizzle2d_perf"

# --- Pytest parametrize for performance testing ---
# Kernel processes one tile of size_i x size_j sequentially.
SWIZZLE_SHAPES_FOR_PERF = [
    # size_i_kernel, size_j_kernel, size_g_kernel
    (32, 32, 4), (64, 64, 8), (128, 128, 16),
    (256, 256, 16), (256, 256, 32),
    (512, 512, 32), #(512, 512, 64) -> size_g should be < size_i, size_j for typical swizzle
    (512, 64, 8), (64, 512, 8)
]
SWIZZLE_DTYPES_FOR_PERF = ['int32', 'int64'] # Output tensor dtype
# NUM_WARPS_FOR_PERF = [1, 2, 4] # Kernel is sequential, num_warps might not have big impact

@pytest.mark.parametrize("size_i_k, size_j_k, size_g_k", SWIZZLE_SHAPES_FOR_PERF)
@pytest.mark.parametrize("output_dtype_str", SWIZZLE_DTYPES_FOR_PERF)
# @pytest.mark.parametrize("num_warps_launch", NUM_WARPS_FOR_PERF)
def test_performance(size_i_k, size_j_k, size_g_k, output_dtype_str, request, device='cuda'):
    # num_warps_launch = 4 # Or from parametrize
    set_seed()
    
    # Ensure size_g is valid for swizzle2d (typically power of 2, < size_i, size_j)
    # tl.swizzle2d doesn't strictly require power of 2 for size_g but it's common.
    # It does require size_g > 0.
    if not (size_g_k > 0 and size_g_k <= size_i_k and size_g_k <= size_j_k):
        pytest.skip(f"Invalid size_g={size_g_k} for tile ({size_i_k}, {size_j_k})")

    if output_dtype_str == 'int64': current_out_dtype = torch.int64
    else: current_out_dtype = torch.int32
        
    # Output buffer. Kernel stores original flattened indices (integers).
    output_buffer = torch.empty((size_i_k * size_j_k,), dtype=current_out_dtype, device=device)
    # The kernel writes to output_ptr + new_i * size_j + new_j.
    # If output_ptr is 1D, this is fine. If output_ptr is 2D, strides are needed.
    # The original test passed a 2D tensor to kernel. Let's match that.
    # Kernel expects output_ptr to be a flat pointer essentially, and calculates 2D offsets.
    # So, a 1D buffer for output_ptr is fine if kernel does `output_ptr + flat_offset`.
    # The kernel does `output_ptr + new_i * size_j + new_j`. This implies `output_ptr` is base of 2D array.
    # So, `output_buffer` should be 2D for the kernel call.
    output_buffer_2d = torch.empty((size_i_k, size_j_k), dtype=current_out_dtype, device=device)


    op_lambda = lambda: swizzle2d_triton_wrapper(
        output_buffer_2d, # Pass the 2D buffer
        size_i_k, size_j_k, size_g_k,
        num_warps_launch=4 
    )

    # This kernel is very fast for small sizes, might need many reps.
    bench_config = do_bench_config(warm_up=100, repetition=1000 if size_i_k*size_j_k < 256*256 else 200) 
    benchmarker = PytestBenchmarker(op_callable=op_lambda,
                                    op_name=OP_NAME_FOR_BENCHMARK,
                                    config=bench_config)
    current_params_for_logs_and_calc = {
        "size_i_kernel": size_i_k, 
        "size_j_kernel": size_j_k,
        "size_g_kernel": size_g_k,
        "output_dtype_str": output_dtype_str,
        "num_warps": 4 
    }
    
    perf_result = benchmarker.run_benchmark(current_params_dict=current_params_for_logs_and_calc,
                                            gbps_calculator=calculate_swizzle2d_gbps,
                                            tflops_calculator=calculate_swizzle2d_tflops)


######################################## HELPERS for Eval ########################################     
# --- Pytest hook to save the dictionary at the end of the session ---  
def test_save_results():  
    """  
    Called after whole test run finished, right before returning the exit status to the system.  
    """
    print('Inside session finish...')
    if "_CALL_SUCCESS_" not in result_gold:
        result_gold['_CALL_SUCCESS_'] = torch.tensor([[0.0]])
    OUTPUT_FILENAME = __file__.replace('.','_') + '.pt'
    print(f"\nSaving all y_triton results to {OUTPUT_FILENAME}...")  
    # Ensure the directory for the output file exists if it's in a subdirectory  
    output_dir = os.path.dirname(OUTPUT_FILENAME)  
    if output_dir and not os.path.exists(output_dir):  
        os.makedirs(output_dir, exist_ok=True)  
    torch.save(result_gold, OUTPUT_FILENAME)       
    print(f"Successfully saved {len(result_gold)} y_triton tensors to {OUTPUT_FILENAME}.")  


def test_save_performance_results():
    """
    Called after the test_performance function finishes.
    This is a separate hook to ensure performance results are saved.
    """
    print('\nPytest session finishing... Saving benchmark results...')

    output_directory = os.path.join(os.path.dirname(__file__), "perf")  # Save in a "perf" subdirectory next to the test file
    os.makedirs(output_directory, exist_ok=True)
    
    save_all_benchmark_results(output_directory)
    print(f"All benchmark results attempted to save to: {output_directory}")


######################################## HELPERS for Eval ########################################


# Modifications Copyright(C)[2025] Advanced Micro Devices, Inc. All rights reserved.
# https://github.com/thunlp/TritonBench - Apache License 2.0
import torch
import triton
import triton.language as tl


@triton.jit
def softmax_kernel(output_ptr, input_ptr, row_stride, n_cols, mask_ptr, BLOCK_SIZE: tl.constexpr):
    r"""the kernel function for implementing softmax operator
    Args:
        output_ptr: the output after finishing softmax operation, (N, hidden_dim)
        input_ptr: the tensor of input, shape should be (N, hidden_dim)
        n_cols(tl.constexpr): the number of cols of input
        BLOCK_SIZE(tl.constexpr): the block_size of your hidden_dim dimension, typically BLOCK_SIZE >= hidden_dim
    """
    row_idx = tl.program_id(0)
    row_start_ptr = input_ptr + row_idx * row_stride
    col_offsets = tl.arange(0, BLOCK_SIZE)
    input_ptrs = row_start_ptr + col_offsets
    row = tl.load(input_ptrs, mask=col_offsets < n_cols, other=-float("inf")).to(tl.float32)
    row_minus_max = row - tl.max(row, axis=0)

    if mask_ptr is not None:
        # load mask into SRAM
        mask_ptrs = (mask_ptr + (row_idx * row_stride)) + col_offsets
        mask = tl.load(mask_ptrs, mask=col_offsets < n_cols, other=0).to(tl.float32)

        # update
        row_minus_max = row_minus_max + mask

    numerator = tl.exp(row_minus_max)
    denominator = tl.sum(numerator, axis=0)
    softmax_output = numerator / denominator
    output_row_start_ptr = output_ptr + row_idx * row_stride
    output_ptrs = output_row_start_ptr + col_offsets
    # Write back output to DRAM
    tl.store(output_ptrs, softmax_output, mask=col_offsets < n_cols)

def softmax(input: torch.Tensor, mask: torch.Tensor = None, dim=-1) -> torch.Tensor:
    if mask is not None:
        # assert input[-1] == mask[-1], "the last dimentions should be the same for input and mask"
        assert input.shape[-1] == mask.shape[-1], "the last dimensions should be the same for input and mask"
    assert dim == -1 or dim == len(input.shape) - 1, "currently softmax layer only support last dimention"

    hidden_dim = input.shape[-1]
    output = torch.empty_like(input)
    input = input.view(-1, hidden_dim)
    if mask is not None:
        mask = mask.view(-1, hidden_dim)
        assert input.shape[0] == mask.shape[0], "the fist dimention of mask and input should be the same"

    num_rows, num_cols = input.shape
    block_size = max(triton.next_power_of_2(num_cols), 2)
    num_warps = 16
    if block_size >= 4096:
        num_warps = 16
    elif block_size >= 2048:
        num_warps = 8
    else:
        num_warps = 4

    if num_rows <= 350000:
        grid = (num_rows,)
        softmax_kernel[grid](
            output, input, input.stride(0), num_cols, mask, BLOCK_SIZE=block_size, num_warps=num_warps
        )
    else:
        grid = lambda meta: (triton.cdiv(num_rows, meta["BLOCK_M"]),)

        softmax_kernel[grid](
            output_ptr=output,
            input_ptr=input,
            row_stride=input.stride(0),
            n_rows=num_rows,
            n_cols=num_cols,
            mask_ptr=mask,
            # currently manually setting up size
            BLOCK_M=32,
            BLOCK_SIZE=block_size,
        )

    return output




##################################################################################################################################################


def test_softmax():
    # Test Case 1: Small matrix without mask
    input_tensor_1 = torch.randn(32, 128, dtype=torch.float16, device='cuda')
    output_tensor_1 = softmax(input_tensor_1)

    # Test Case 2: Small matrix with mask
    input_tensor_2 = torch.randn(32, 128, dtype=torch.float16, device='cuda')
    mask_tensor_2 = torch.randint(0, 2, (32, 128), dtype=torch.float16, device='cuda')
    output_tensor_2 = softmax(input_tensor_2, mask=mask_tensor_2)

    # Test Case 3: Larger matrix without mask
    input_tensor_3 = torch.randn(1024, 512, dtype=torch.float16, device='cuda')
    output_tensor_3 = softmax(input_tensor_3)

    # Test Case 4: Larger matrix with mask
    input_tensor_4 = torch.randn(1024, 512, dtype=torch.float16, device='cuda')
    mask_tensor_4 = torch.randint(0, 2, (1024, 512), dtype=torch.float16, device='cuda')
    output_tensor_4 = softmax(input_tensor_4, mask=mask_tensor_4)

    # Test Case 5: Very large matrix without mask
    input_tensor_5 = torch.randn(100000, 256, dtype=torch.float16, device='cuda')
    output_tensor_5 = softmax(input_tensor_5)

    # Test Case 6: Very large matrix with mask
    input_tensor_6 = torch.randn(100000, 256, dtype=torch.float16, device='cuda')
    mask_tensor_6 = torch.randint(0, 2, (100000, 256), dtype=torch.float16, device='cuda')
    output_tensor_6 = softmax(input_tensor_6, mask=mask_tensor_6)

    return {
        "test_case_1": output_tensor_1,
        "test_case_2": output_tensor_2,
        "test_case_3": output_tensor_3,
        "test_case_4": output_tensor_4,
        "test_case_5": output_tensor_5,
        "test_case_6": output_tensor_6
    }

# Run the test function
result_gold = test_softmax()



import sys
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from TritonBench_v1.chunk_linear_attn import chunk_linear_attn
from performance_utils import Performance_Metrics, do_bench_config

import torch
import triton
import triton.language as tl

class performance_metrics(Performance_Metrics):
    def __init__(self, dtype=None, is_backward=False, **kwargs):
        super().__init__('chunk_linear_attn', dtype=dtype, is_backward=is_backward, **kwargs)
        
    def get_input_tensors(self):
        self.input_tensors = []
        for i in range(2, 12):  # Adjust the range as needed
            B, H, T, K, V = 32, 128, 2 ** i, 64, 64  # Example dimensions
            q = torch.rand(B, H, T, K, dtype=torch.float32)
            k = torch.rand(B, H, T, K, dtype=torch.float32)
            v = torch.rand(B, H, T, V, dtype=torch.float32)
            self.input_tensors.append((q, k, v))

    def to_cuda(self, input_tensor):
        q, k, v = input_tensor
        return q.cuda(), k.cuda(), v.cuda()

    def call_op(self, input_tensor):
        q, k, v = input_tensor
        scale = q.shape[-1] ** -0.5
        return chunk_linear_attn(q, k, v, scale=scale)

    def get_gbps(self, input_tensor, runtime):
        q, k, v = input_tensor
        B, H, T, K, V = q.shape[0], q.shape[1], q.shape[2], q.shape[3], v.shape[3]
        total_bytes = (q.numel() + k.numel() + v.numel() + B * H * T * V) * q.element_size()
        GBPS = total_bytes / (runtime / 1000) / 1e9
        return GBPS
    
    def get_tflops(self, input_tensor, runtime):
        q, k, v = input_tensor
        B, H, T, K, V = q.shape[0], q.shape[1], q.shape[2], q.shape[3], v.shape[3]
        FLOPS = 2 * B * H * T * K * V  # Simplified FLOPS calculation
        TFLOPS = FLOPS / (runtime / 1000) / 1e12
        return TFLOPS

if __name__ == '__main__':
    op_perf = performance_metrics()
    op_perf.get_input_tensors()
    op_perf.get_do_bench_config(warmup=100, rep=1000)
    op_perf.run_benchmark()



import sys
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from TritonBench_v1.parallel_retention_attention import parallel_retention
from performance_utils import Performance_Metrics, do_bench_config

import torch
import triton
import triton.language as tl

class performance_metrics(Performance_Metrics):
    def __init__(self, dtype=None, is_backward=False, **kwargs):
        super().__init__('parallel_retention_attention', dtype=dtype, is_backward=is_backward, **kwargs)
        
    def get_input_tensors(self):
        for i in range(2, 18):  # Adjust the range as needed for your tests
            B = 2  # Batch size
            H = 8  # Number of heads
            T = 2 ** i  # Sequence length
            D_head_K = 64  # Dimension of head for Q/K
            D_head_V = 64  # Dimension of head for V
            q = torch.rand(B, H, T, D_head_K, dtype=torch.float16)
            k = torch.rand(B, H, T, D_head_K, dtype=torch.float16)
            v = torch.rand(B, H, T, D_head_V, dtype=torch.float16)
            self.input_tensors.append((q, k, v))

    def to_cuda(self, input_tensor):
        return tuple(tensor.cuda() for tensor in input_tensor)

    def call_op(self, input_tensor):
        q, k, v = input_tensor
        return parallel_retention(q, k, v)

    def get_gbps(self, input_tensor, runtime):
        q, k, v = input_tensor
        total_bytes = (q.numel() + k.numel() + v.numel()) * q.element_size()
        GBPS = total_bytes / (runtime / 1000) / 1e9
        return GBPS
    
    def get_tflops(self, input_tensor, runtime):
        q, k, v = input_tensor
        B, H, T, D_head_K = q.shape
        D_head_V = v.shape[-1]
        # Assuming FLOPS is calculated as 2 * B * H * T * D_head_K * D_head_V
        FLOPS = 2 * B * H * T * D_head_K * D_head_V
        TFLOPS = FLOPS / (runtime / 1000) / 1e12
        return TFLOPS

if __name__ == '__main__':
    op_perf = performance_metrics()
    op_perf.get_input_tensors()
    op_perf.get_do_bench_config(warmup=100, rep=1000)
    op_perf.run_benchmark()



import sys
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from TritonBench_v1.chunked_cumsum_fwd import _chunk_cumsum_fwd
from performance_utils import Performance_Metrics, do_bench_config

import torch
import triton
import triton.language as tl

class performance_metrics(Performance_Metrics):
    def __init__(self, dtype=None, is_backward=False, **kwargs):
        super().__init__('chunked_cumsum_fwd', dtype=dtype, is_backward=is_backward, **kwargs)
        
    def get_input_tensors(self):
        self.input_tensors = []
        for i in range(3, 11):  # Adjust the range for different sizes
            batch = 128 * i
            seqlen = 128 * i
            nheads = 128 * (i // 2)
            chunk_size = 128 * (i // 3)
            dt = torch.rand(batch, seqlen, nheads, dtype=torch.float32)
            A = torch.rand(nheads, dtype=torch.float32)
            dt_bias = torch.rand(nheads, dtype=torch.float32)
            self.input_tensors.append((dt, A, chunk_size, dt_bias))

    def to_cuda(self, input_tensor):
        dt, A, chunk_size, dt_bias = input_tensor
        return (dt.cuda(), A.cuda(), chunk_size, dt_bias.cuda())

    def call_op(self, input_tensor):
        dt, A, chunk_size, dt_bias = input_tensor
        return _chunk_cumsum_fwd(dt, A, chunk_size, dt_bias)

    def get_gbps(self, input_tensor, runtime):
        dt, A, chunk_size, dt_bias = input_tensor
        total_bytes = dt.numel() * dt.element_size() + A.numel() * A.element_size() + dt_bias.numel() * dt_bias.element_size()
        GBPS = total_bytes / (runtime / 1000) / 1e9
        return GBPS
    
    def get_tflops(self, input_tensor, runtime):
        dt, A, chunk_size, dt_bias = input_tensor
        batch, seqlen, nheads = dt.shape
        FLOPS = batch * seqlen * nheads * 2  # Assuming 2 operations per element
        TFLOPS = FLOPS / (runtime / 1000) / 1e12
        return TFLOPS

if __name__ == '__main__':
    op_perf = performance_metrics()
    op_perf.get_input_tensors()
    op_perf.get_do_bench_config(warmup=100, rep=1000)
    op_perf.run_benchmark()



import sys
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from TritonBench_v1.layer_norm_welfold import fused_native_layer_norm_no_welford
from performance_utils import Performance_Metrics, do_bench_config

import torch
import triton
import triton.language as tl

class performance_metrics(Performance_Metrics):
    def __init__(self, dtype=None, is_backward=False, **kwargs):
        super().__init__('layer_norm_welfold', dtype=dtype, is_backward=is_backward, **kwargs)
        
    def get_input_tensors(self):
        for i in range(4, 20):  # Adjust the range for desired sizes
            S = 2 ** i
            D = 1024  # Fixed dimension size for testing
            primals_1 = torch.rand(D, dtype=torch.float32)
            primals_2 = torch.rand(D, dtype=torch.float32)
            primals_3 = torch.rand(S, D, dtype=torch.bfloat16)
            self.input_tensors.append((primals_1, primals_2, primals_3))

    def to_cuda(self, input_tensor):
        return tuple(tensor.cuda() for tensor in input_tensor)

    def call_op(self, input_tensor):
        return fused_native_layer_norm_no_welford(*input_tensor)

    def get_gbps(self, input_tensor, runtime):
        primals_3 = input_tensor[2]
        total_bytes = (primals_3.numel() * primals_3.element_size() * 2)
        GBPS = total_bytes / (runtime / 1000) / 1e9
        return GBPS
    
    def get_tflops(self, input_tensor, runtime):
        primals_3 = input_tensor[2]
        FLOPS = 2 * primals_3.numel()  # Assuming 2 FLOPS per element
        TFLOPS = FLOPS / (runtime / 1000) / 1e12
        return TFLOPS

if __name__ == '__main__':
    op_perf = performance_metrics()
    op_perf.get_input_tensors()
    op_perf.get_do_bench_config(warmup=100, rep=1000)
    op_perf.run_benchmark()



# Modifications Copyright(C)[2025] Advanced Micro Devices, Inc. All rights reserved.
# https://github.com/thunlp/TritonBench - Apache License 2.0
import logging

import torch
import triton
import triton.language as tl
import functools
import operator
from typing import Tuple

Shape = Tuple[int]


try:
    uint_to_uniform_float = tl.uint_to_uniform_float
except AttributeError:
    # Copied from triton.language package for compatibility
    @triton.jit
    def uint_to_uniform_float(x):
        """
        Numerically stable function to convert a random uint into a random float uniformly sampled in [0, 1).
        """
        # TODO: fix frontend issues and cleanup
        # conditions can be simplified
        # scale is ((2**23 - 1) / 2**23) * 2**(N_BITS - 1)
        if tl.constexpr(x.dtype == tl.uint32) or tl.constexpr(x.dtype == tl.int32):
            # maximum value such that `MAX_INT * scale < 1.0` (with float rounding)
            x = x.to(tl.int32, bitcast=True)
            scale = 4.6566127342e-10
        else:
            tl.static_assert(
                tl.constexpr(x.dtype == tl.uint64) or tl.constexpr(x.dtype == tl.int64)
            )
            x = x.to(tl.int64, bitcast=True)
            scale = 1.0842020432385337e-19
        x = tl.where(x < 0, -x - 1, x)
        return x * scale


def philox_cuda_seed_offset(increment, device=None):
    device = device or torch.cuda.current_device()
    gen = torch.cuda.default_generators[device]
    state_copy = gen.get_state()
    c0, c1 = state_copy.view(torch.int64)
    seed, offset = int(c0), int(c1)
    increment = (increment + 3) // 4 * 4
    c1 += increment
    # get_state returns a new tensor, so it needs set_state to update the actual generator state.
    gen.set_state(state_copy)
    return seed, offset


def heur_block(args):
    if args["N"] <= 512:
        return 512
    else:
        return 1024


def heur_num_warps(args):
    if args["N"] <= 512:
        return 4
    elif args["N"] <= 1024:
        return 8
    else:
        return 16


@triton.heuristics(
    {
        "BLOCK": heur_block,
        "num_warps": heur_num_warps,
    }
)
@triton.jit(do_not_specialize=["philox_seed", "philox_offset"])
def uniform_kernel(
    out_ptr,
    N,
    philox_seed,
    philox_offset,
    from_,
    to,
    BLOCK: tl.constexpr,
):
    philox_seed = philox_seed.to(tl.int64)
    philox_offset = philox_offset.to(tl.int64)
    c0 = (philox_offset & 0xFFFFFFFF).to(tl.uint32)
    c1 = ((philox_offset >> 32) & 0xFFFFFFFF).to(tl.uint32)
    i4 = tl.program_id(0) * BLOCK + tl.arange(0, BLOCK)
    c0 += i4
    _O = c0 * 0
    r0, r1, r2, r3 = tl.philox(philox_seed, c0, c1, _O, _O)
    r0 = uint_to_uniform_float(r0) * (to - from_) + from_
    r1 = uint_to_uniform_float(r1) * (to - from_) + from_
    r2 = uint_to_uniform_float(r2) * (to - from_) + from_
    r3 = uint_to_uniform_float(r3) * (to - from_) + from_
    off_0 = tl.program_id(0) * BLOCK * 4 + tl.arange(0, BLOCK)
    off_1 = off_0 + BLOCK
    off_2 = off_1 + BLOCK
    off_3 = off_2 + BLOCK
    tl.store(out_ptr + off_0, r0, mask=off_0 < N, eviction_policy="evict_first")
    tl.store(out_ptr + off_1, r1, mask=off_1 < N, eviction_policy="evict_first")
    tl.store(out_ptr + off_2, r2, mask=off_2 < N, eviction_policy="evict_first")
    tl.store(out_ptr + off_3, r3, mask=off_3 < N, eviction_policy="evict_first")



def volume(shape: Shape) -> int:
    return functools.reduce(operator.mul, shape, 1)


UNROLL = 4


def uniform_(self, from_=0.0, to=1.0, *, generator=None):
    logging.debug("GEMS UNIFORM")
    N = volume(self.shape)
    grid_fn = lambda meta: (triton.cdiv(N, meta["BLOCK"] * UNROLL),)

    increment = triton.cdiv(N, UNROLL)
    philox_seed, philox_offset = philox_cuda_seed_offset(increment)
    with torch.cuda.device(self.device):
        uniform_kernel[grid_fn](self, N, philox_seed, philox_offset, from_, to)
    return self




##################################################################################################################################################


def test_uniform_():
    results = {}

    # Test case 1: 1D tensor, float32, default from=0, to=1
    x_1d = torch.empty((10,), device='cuda', dtype=torch.float32)
    uniform_(x_1d)
    results['test_case_1'] = x_1d

    # Test case 2: 2D tensor, float32, from=2, to=5
    x_2d = torch.empty((4, 4), device='cuda', dtype=torch.float32)
    uniform_(x_2d, from_=2.0, to=5.0)
    results['test_case_2'] = x_2d

    # Test case 3: 3D tensor, float64, from=-1, to=1
    x_3d = torch.empty((2, 3, 4), device='cuda', dtype=torch.float64)
    uniform_(x_3d, from_=-1.0, to=1.0)
    results['test_case_3'] = x_3d

    # Test case 4: Empty tensor, float32, from=0, to=1
    x_empty = torch.empty((0,), device='cuda', dtype=torch.float32)
    uniform_(x_empty)
    results['test_case_4'] = x_empty

    # Test case 5: Single-element tensor, float32, from=5, to=10
    x_single = torch.empty((1,), device='cuda', dtype=torch.float32)
    uniform_(x_single, from_=5.0, to=10.0)
    results['test_case_5'] = x_single

    # Test case 6: Large tensor, float32, from=-10, to=10
    x_large = torch.empty((1024, 1024), device='cuda', dtype=torch.float32)
    uniform_(x_large, from_=-10.0, to=10.0)
    results['test_case_6'] = x_large

    # Test case 7: 1D tensor, float16, default from=0, to=1
    x_float16 = torch.empty((10,), device='cuda', dtype=torch.float16)
    uniform_(x_float16)
    results['test_case_7'] = x_float16

    # Test case 8: Tensor with specific shapes (non-square), float32, from=-5, to=5
    x_shape1 = torch.empty((3, 7), device='cuda', dtype=torch.float32)
    x_shape2 = torch.empty((5, 3, 2), device='cuda', dtype=torch.float32)
    uniform_(x_shape1, from_=-5.0, to=5.0)
    uniform_(x_shape2, from_=-5.0, to=5.0)
    results['test_case_8_1'] = x_shape1
    results['test_case_8_2'] = x_shape2

    # Test case 9: Tensor with 4D shape, float32, from=0.5, to=2.5
    x_4d = torch.empty((2, 2, 3, 4), device='cuda', dtype=torch.float32)
    uniform_(x_4d, from_=0.5, to=2.5)
    results['test_case_9'] = x_4d

    # Test case 10: Tensor with nan values, float32, checking if the range is filled correctly
    x_nan = torch.full((10,), float('nan'), device='cuda', dtype=torch.float32)
    uniform_(x_nan, from_=1.0, to=2.0)
    results['test_case_10'] = x_nan

    # Test case 11: Tensor with negative shape values (expecting an exception)
    try:
        x_negative = torch.empty((-1,), device='cuda', dtype=torch.float32)
        uniform_(x_negative)
    except Exception as e:
        results['test_case_11'] = str(e)

    # Test case 12: Tensor with mixed positive and negative shape values (expecting an exception)
    try:
        x_invalid = torch.empty((3, -4), device='cuda', dtype=torch.float32)
        uniform_(x_invalid)
    except Exception as e:
        results['test_case_12'] = str(e)

    # Test case 13: Very large tensor, checking if it handles allocation and computation
    x_very_large = torch.empty((4096, 4096), device='cuda', dtype=torch.float32)
    uniform_(x_very_large, from_=-10.0, to=10.0)
    results['test_case_13'] = x_very_large

    # Test case 14: Tensor with extreme value ranges
    x_extreme = torch.empty((10,), device='cuda', dtype=torch.float32)
    uniform_(x_extreme, from_=-1e5, to=1e5)
    results['test_case_14'] = x_extreme

    # Test case 15: Edge case where from_ == to
    x_equal = torch.empty((10,), device='cuda', dtype=torch.float32)
    uniform_(x_equal, from_=5.0, to=5.0)
    results['test_case_15'] = x_equal

    # Test case 16: Tensor with NaN, Inf values for from_ and to (expecting exception)
    try:
        x_nan_inf = torch.empty((10,), device='cuda', dtype=torch.float32)
        uniform_(x_nan_inf, from_=float('nan'), to=float('inf'))
    except Exception as e:
        results['test_case_16'] = str(e)

    return results

result_gold = test_uniform_()



import sys
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from TritonBench_v1.token_attn_reduceV import token_att_fwd2
from performance_utils import Performance_Metrics, do_bench_config

import torch
import triton
import triton.language as tl

class performance_metrics(Performance_Metrics):
    def __init__(self, dtype=None, is_backward=False, **kwargs):
        super().__init__('token_attn_reduceV', dtype=dtype, is_backward=is_backward, **kwargs)
        
    def get_input_tensors(self):
        self.input_tensors = []
        for i in range(2, 16):  # Adjust the range as needed for your testing
            batch_size = 2 ** i
            seq_len = 128  # Example sequence length
            num_heads = 8
            d_model = 64  # Example model dimension

            prob = torch.rand((num_heads, seq_len), dtype=torch.float16)
            v = torch.rand((num_heads, seq_len, d_model), dtype=torch.float16)
            out = torch.zeros((batch_size, num_heads, d_model), dtype=torch.float16)
            Req_to_tokens = torch.randint(0, seq_len, (batch_size, seq_len), dtype=torch.int32)
            B_req_idx = torch.arange(batch_size, dtype=torch.int32)
            B_Start_Loc = torch.zeros(batch_size, dtype=torch.int32)
            B_Seqlen = torch.full((batch_size,), seq_len, dtype=torch.int32)

            self.input_tensors.append((prob, v, out, Req_to_tokens, B_req_idx, B_Start_Loc, B_Seqlen))

    def to_cuda(self, input_tensor):
        return tuple(tensor.cuda() for tensor in input_tensor)

    def call_op(self, input_tensor):
        prob, v, out, Req_to_tokens, B_req_idx, B_Start_Loc, B_Seqlen = input_tensor
        token_att_fwd2(prob, v, out, Req_to_tokens, B_req_idx, B_Start_Loc, B_Seqlen)
        return out

    def get_gbps(self, input_tensor, runtime):
        prob, v, out, Req_to_tokens, B_req_idx, B_Start_Loc, B_Seqlen = input_tensor
        total_bytes = (prob.numel() + v.numel() + out.numel() + Req_to_tokens.numel() + B_req_idx.numel() + B_Start_Loc.numel() + B_Seqlen.numel()) * prob.element_size()
        GBPS = total_bytes / (runtime / 1000) / 1e9
        return GBPS
    
    def get_tflops(self, input_tensor, runtime):
        prob, v, out, Req_to_tokens, B_req_idx, B_Start_Loc, B_Seqlen = input_tensor
        # Assuming each element in the output is the result of a dot product
        FLOPS = 2 * prob.size(0) * prob.size(1) * v.size(2)
        TFLOPS = FLOPS / (runtime / 1000) / 1e12
        return TFLOPS

if __name__ == '__main__':
    op_perf = performance_metrics()
    op_perf.get_input_tensors()
    op_perf.get_do_bench_config(warmup=100, rep=1000)
    op_perf.run_benchmark()



import sys
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from TritonBench_v1.flash_decode2_phi import flash_decode_stage2
from performance_utils import Performance_Metrics, do_bench_config

import torch
import triton
import triton.language as tl

class performance_metrics(Performance_Metrics):
    def __init__(self, dtype=None, is_backward=False, **kwargs):
        super().__init__('flash_decode2_phi', dtype=dtype, is_backward=is_backward, **kwargs)
        
    def get_input_tensors(self):
        self.input_tensors = []
        for i in range(2, 18):  # Adjust the range as needed for your testing
            batch_size = 2 ** i
            head_num = 8  # Example head number, adjust as needed
            seq_block_num = 16  # Example sequence block number, adjust as needed
            head_dim = 64  # Example head dimension, adjust as needed

            mid_out = torch.rand(batch_size, head_num, seq_block_num, head_dim, dtype=torch.float32)
            mid_out_logexpsum = torch.rand(batch_size, head_num, seq_block_num, dtype=torch.float32)
            B_Seqlen = torch.randint(1, seq_block_num * 32, (batch_size,), dtype=torch.int32)
            Out = torch.empty(batch_size, head_num, head_dim, dtype=torch.float32)

            self.input_tensors.append((mid_out, mid_out_logexpsum, B_Seqlen, Out))

    def to_cuda(self, input_tensor):
        mid_out, mid_out_logexpsum, B_Seqlen, Out = input_tensor
        return (mid_out.cuda(), mid_out_logexpsum.cuda(), B_Seqlen.cuda(), Out.cuda())

    def call_op(self, input_tensor):
        mid_out, mid_out_logexpsum, B_Seqlen, Out = input_tensor
        block_seq = 32  # Example block sequence size, adjust as needed
        flash_decode_stage2(mid_out, mid_out_logexpsum, B_Seqlen, Out, block_seq)
        return Out

    def get_gbps(self, input_tensor, runtime):
        mid_out, mid_out_logexpsum, B_Seqlen, Out = input_tensor
        total_bytes = mid_out.numel() * mid_out.element_size() + mid_out_logexpsum.numel() * mid_out_logexpsum.element_size() + B_Seqlen.numel() * B_Seqlen.element_size() + Out.numel() * Out.element_size()
        GBPS = total_bytes / (runtime / 1000) / 1e9
        return GBPS
    
    def get_tflops(self, input_tensor, runtime):
        mid_out, _, _, _ = input_tensor
        FLOPS = 2 * mid_out.numel()  # Example calculation, adjust based on actual operations
        TFLOPS = FLOPS / (runtime / 1000) / 1e12
        return TFLOPS

if __name__ == '__main__':
    op_perf = performance_metrics()
    op_perf.get_input_tensors()
    op_perf.get_do_bench_config(warmup=100, rep=1000)
    op_perf.run_benchmark()
